@@ -37,10 +37,12 @@ struct grassvert
 {
     vec pos;
     bvec4 color;
-    float u, v, lmu, lmv;
+    vec2 tc, lm;
 };
 
 static vector<grassvert> grassverts;
+static GLuint grassvbo = 0;
+static int grassvbosize = 0;
 
 struct grassgroup
 {
@@ -108,9 +110,9 @@ static void gengrassquads(grassgroup *&group, const grasswedge &w, const grasstr
     vec tc;
     tc.cross(g.surface, w.dir).mul(texscale);
 
-    int color = tstep + maxstep;
-    if(color < 0) color = numgrassoffsets - (-color)%numgrassoffsets;
-    color += numsteps + numgrassoffsets - numsteps%numgrassoffsets;
+    int offset = tstep + maxstep;
+    if(offset < 0) offset = numgrassoffsets - (-offset)%numgrassoffsets;
+    offset += numsteps + numgrassoffsets - numsteps%numgrassoffsets;
 
     float leftdist = t0;
     const vec *leftv = &g.v[0];
@@ -127,7 +129,7 @@ static void gengrassquads(grassgroup *&group, const grasswedge &w, const grasstr
           leftb = 0, rightb = 0, leftdb = 0, rightdb = 0;
     bvec gcol = col.iszero() ? grasscolor : bvec(uchar(col.x*255), uchar(col.y*255), uchar(col.z*255));
     if(blend <= 0) blend = grassblend;
-    for(int i = maxstep; i >= minstep; i--, color--, leftp.add(leftdir), rightp.add(rightdir), leftb += leftdb, rightb += rightdb, dist -= grassstep)
+    for(int i = maxstep; i >= minstep; i--, offset--, leftp.add(leftdir), rightp.add(rightdir), leftb += leftdb, rightb += rightdb, dist -= grassstep)
     {
         if(dist <= leftdist)
         {
@@ -187,7 +189,7 @@ static void gengrassquads(grassgroup *&group, const grasswedge &w, const grasstr
             extern bool brightengeom;
             int lmid = brightengeom && (g.lmid < LMID_RESERVED || (fullbright && editmode)) ? LMID_BRIGHT : g.lmid;
             group->lmtex = lightmaptexs.inrange(lmid) ? lightmaptexs[lmid].id : notexture->id;
-            group->offset = grassverts.length();
+            group->offset = grassverts.length()/4;
             group->numquads = 0;
             group->scale = gs;
             group->height = gh;
@@ -196,26 +198,26 @@ static void gengrassquads(grassgroup *&group, const grasswedge &w, const grasstr
 
         group->numquads++;
 
-        float offset = grassoffsets[color%numgrassoffsets],
-              animoffset = animscale*grassanimoffsets[color%numgrassoffsets],
-              tc1 = tc.dot(p1) + offset, tc2 = tc.dot(p2) + offset,
-              lm1u = g.tcu.dot(p1), lm1v = g.tcv.dot(p1),
-              lm2u = g.tcu.dot(p2), lm2v = g.tcv.dot(p2),
+        float tcoffset = grassoffsets[offset%numgrassoffsets],
+              animoffset = animscale*grassanimoffsets[offset%numgrassoffsets],
+              tc1 = tc.dot(p1) + tcoffset, tc2 = tc.dot(p2) + tcoffset,
               fade = dist - t > taperdist ? (grassdist - (dist - t))*taperscale : 1,
               height = gh * fade;
+        vec2 lm1(g.tcu.dot(p1), g.tcv.dot(p1)),
+             lm2(g.tcu.dot(p2), g.tcv.dot(p2));
         bvec4 color(gcol, uchar(fade*blend*255));
 
         #define GRASSVERT(n, tcv, modify) { \
             grassvert &gv = grassverts.add(); \
             gv.pos = p##n; \
             gv.color = color; \
-            gv.u = tc##n; gv.v = tcv; \
-            gv.lmu = lm##n##u; gv.lmv = lm##n##v; \
+            gv.tc = vec2(tc##n, tcv); \
+            gv.lm = lm##n; \
             modify; \
         }
 
-        GRASSVERT(2, 0, { gv.pos.z += height; gv.u += animoffset; });
-        GRASSVERT(1, 0, { gv.pos.z += height; gv.u += animoffset; });
+        GRASSVERT(2, 0, { gv.pos.z += height; gv.tc.x += animoffset; });
+        GRASSVERT(1, 0, { gv.pos.z += height; gv.tc.x += animoffset; });
         GRASSVERT(1, 1, );
         GRASSVERT(2, 1, );
     }
@@ -234,7 +236,7 @@ static void gengrassquads(vtxarray *va)
         if(!s.grasstex)
         {
             if(!s.texgrass) continue;
-            s.grasstex = textureload(makerelpath(NULL, s.texgrass, NULL, "<ffskip><premul>"), 2);
+            s.grasstex = textureload(makerelpath(NULL, s.texgrass, NULL, "<premul>"), 2);
         }
 
         grassgroup *group = NULL;
@@ -269,7 +271,6 @@ void generategrass()
         w.bound2.offset = -camera1->o.dot(w.bound2);
     }
 
-    extern vtxarray *visibleva;
     for(vtxarray *va = visibleva; va; va = va->next)
     {
         if(va->grasstris.empty() || va->occluded >= OCCLUDE_GEOM) continue;
@@ -278,7 +279,17 @@ void generategrass()
         gengrassquads(va);
     }
 
+    if(grassgroups.empty()) return;
+
     grassgroups.sort(comparegrassgroups);
+
+    if(!grassvbo) glGenBuffers_(1, &grassvbo);
+    gle::bindvbo(grassvbo);
+    int size = grassverts.length()*sizeof(grassvert);
+    grassvbosize = max(grassvbosize, size);
+    glBufferData_(GL_ARRAY_BUFFER, grassvbosize, size == grassvbosize ? grassverts.getbuf() : NULL, GL_STREAM_DRAW);
+    if(size != grassvbosize) glBufferSubData_(GL_ARRAY_BUFFER, 0, size, grassverts.getbuf());
+    gle::clearvbo();
 }
 
 void rendergrass()
@@ -287,31 +298,23 @@ void rendergrass()
 
     glDisable(GL_CULL_FACE);
     glEnable(GL_BLEND);
-    glBlendFunc(renderpath==R_FIXEDFUNCTION ? GL_SRC_ALPHA : GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
     glDepthMask(GL_FALSE);
 
     SETSHADER(grass);
 
-    glEnableClientState(GL_VERTEX_ARRAY);
-    glVertexPointer(3, GL_FLOAT, sizeof(grassvert), grassverts[0].pos.v);
+    gle::bindvbo(grassvbo);
 
-    glEnableClientState(GL_COLOR_ARRAY);
-    glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(grassvert), grassverts[0].color.v);
-
-    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-    glTexCoordPointer(2, GL_FLOAT, sizeof(grassvert), &grassverts[0].u);
-
-    if(renderpath!=R_FIXEDFUNCTION || maxtmus>=2)
-    {
-        glActiveTexture_(GL_TEXTURE1_ARB);
-        glClientActiveTexture_(GL_TEXTURE1_ARB);
-        glEnable(GL_TEXTURE_2D);
-        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-        glTexCoordPointer(2, GL_FLOAT, sizeof(grassvert), &grassverts[0].lmu);
-        if(renderpath==R_FIXEDFUNCTION) setuptmu(1, "P * T x 2");
-        glClientActiveTexture_(GL_TEXTURE0_ARB);
-        glActiveTexture_(GL_TEXTURE0_ARB);
-    }
+    const grassvert *ptr = 0;
+    gle::vertexpointer(sizeof(grassvert), ptr->pos.v);
+    gle::colorpointer(sizeof(grassvert), ptr->color.v);
+    gle::texcoord0pointer(sizeof(grassvert), ptr->tc.v);
+    gle::texcoord1pointer(sizeof(grassvert), ptr->lm.v);
+    gle::enablevertex();
+    gle::enablecolor();
+    gle::enabletexcoord0();
+    gle::enabletexcoord1();
+    gle::enablequads();
 
     int texid = -1, lmtexid = -1;
     loopv(grassgroups)
@@ -331,36 +334,33 @@ void rendergrass()
         }
         if(lmtexid != g.lmtex)
         {
-            if(renderpath!=R_FIXEDFUNCTION || maxtmus>=2)
-            {
-                glActiveTexture_(GL_TEXTURE1_ARB);
-                glBindTexture(GL_TEXTURE_2D, g.lmtex);
-                glActiveTexture_(GL_TEXTURE0_ARB);
-            }
+            glActiveTexture_(GL_TEXTURE1);
+            glBindTexture(GL_TEXTURE_2D, g.lmtex);
+            glActiveTexture_(GL_TEXTURE0);
             lmtexid = g.lmtex;
         }
 
-        glDrawArrays(GL_QUADS, g.offset, 4*g.numquads);
+        gle::drawquads(g.offset, g.numquads);
         xtravertsva += 4*g.numquads;
     }
 
-    glDisableClientState(GL_VERTEX_ARRAY);
-    glDisableClientState(GL_COLOR_ARRAY);
-    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-
-    if(renderpath!=R_FIXEDFUNCTION || maxtmus>=2)
-    {
-        glActiveTexture_(GL_TEXTURE1_ARB);
-        glClientActiveTexture_(GL_TEXTURE1_ARB);
-        if(renderpath==R_FIXEDFUNCTION) resettmu(1);
-        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-        glDisable(GL_TEXTURE_2D);
-        glClientActiveTexture_(GL_TEXTURE0_ARB);
-        glActiveTexture_(GL_TEXTURE0_ARB);
-    }
+    gle::disablequads();
+    gle::disablevertex();
+    gle::disablecolor();
+    gle::disabletexcoord0();
+    gle::disabletexcoord1();
+    
+    gle::clearvbo();
 
     glDisable(GL_BLEND);
     glDepthMask(GL_TRUE);
     glEnable(GL_CULL_FACE);
 }
 
+void cleanupgrass()
+{
+    if(grassvbo) { glDeleteBuffers_(1, &grassvbo); grassvbo = 0; }
+    grassvbosize = 0;
+}
+
+
