@@ -2,14 +2,25 @@
 // runs dedicated or as client coroutine
 
 #include "engine.h"
+#include <signal.h>
 #ifdef WIN32
 #include <shlobj.h>
 #endif
 
+int curtime = 0, totalmillis = 1, lastmillis = 1, timescale = 100, paused = 0, timeerr = 0, shutdownwait = 0;
+time_t clocktime = 0, currenttime = 0, clockoffset = 0;
+uint totalsecs = 0;
+
+VAR(0, maxruntime, 0, (INT_MAX-1)/1000, VAR_MAX); // time in seconds
+VAR(0, maxshutdownwait, 0, 3600, VAR_MAX); // time in seconds
+
+const char *load = NULL;
+vector<char *> gameargs;
+
 const char *platnames[MAX_PLATFORMS] = {
-    "win", "osx", "nix"
+    "win", "mac", "nix"
 }, *platlongnames[MAX_PLATFORMS] = {
-    "windows", "macosx", "linux/bsd"
+    "windows", "macos", "linux/bsd"
 };
 
 VAR(0, versioning, 1, 0, -1);
@@ -21,6 +32,7 @@ VAR(IDF_READONLY, versionpatch, 1, VERSION_PATCH, -1);
 SVAR(IDF_READONLY, versionstring, VERSION_STRING);
 SVAR(IDF_READONLY, versionname, VERSION_NAME);
 SVAR(IDF_READONLY, versionuname, VERSION_UNAME);
+SVAR(IDF_READONLY, versionvname, VERSION_VNAME);
 SVAR(IDF_READONLY, versionrelease, VERSION_RELEASE);
 SVAR(IDF_READONLY, versionurl, VERSION_URL);
 SVAR(IDF_READONLY, versioncopy, VERSION_COPY);
@@ -29,35 +41,40 @@ SVAR(IDF_READONLY, versionplatname, plat_name(CUR_PLATFORM));
 SVAR(IDF_READONLY, versionplatlongname, plat_longname(CUR_PLATFORM));
 VAR(IDF_READONLY, versionplatform, 0, CUR_PLATFORM, VAR_MAX);
 VAR(IDF_READONLY, versionarch, 0, CUR_ARCH, VAR_MAX);
+VAR(IDF_READONLY, versioncrc, 0, 0, VAR_MAX);
+SVAR(IDF_READONLY, versionbranch, "none");
 #ifdef STANDALONE
 VAR(IDF_READONLY, versionisserver, 0, 1, 1);
 #else
 VAR(IDF_READONLY, versionisserver, 0, 0, 1);
 #endif
-uint versioncrc = 0;
 ICOMMAND(0, platname, "ii", (int *p, int *g), result(*p >= 0 && *p < MAX_PLATFORMS ? (*g!=0 ? plat_longname(*p) : plat_name(*p)) : ""));
 
+SVAR(IDF_READONLY, systemuser, "none");
+SVAR(IDF_READONLY, systemhost, "unknown");
+
 VAR(0, rehashing, 1, 0, -1);
 
-const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is password protected", "server requires pure official builds", "server is at maximum capacity", "server and client are incompatible", "connection timed out", "packet overflow", "server shutting down" };
+const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is password protected", "server requires pure official builds", "server is at maximum capacity", "server and client are incompatible", "connection timed out", "packet overflow", "hostname lookup failure", "server shutting down" };
 
 SVAR(IDF_PERSIST, logtimeformat, "%Y-%m-%d %H:%M.%S");
+VAR(IDF_PERSIST, logtimelocal, 0, 1, 1); // use clockoffset to localise
 SVAR(IDF_PERSIST, filetimeformat, "%Y%m%d%H%M%S");
 VAR(IDF_PERSIST, filetimelocal, 0, 1, 1); // use clockoffset to localise
 
 const char *gettime(time_t ctime, const char *format)
 {
     static string buf;
-    if(!ctime) ctime = currenttime;
+    if(!ctime) ctime = logtimelocal ? currenttime : clocktime;
     struct tm *t = localtime(&ctime);
     if(!strftime(buf, sizeof(buf), format && *format ? format : logtimeformat, t)) buf[0] = '\0';
     return buf;
 }
-ICOMMAND(0, gettime, "is", (int *n, char *a), result(gettime(*n, a)));
+ICOMMAND(0, gettime, "isi", (int *n, char *a, int *p), result(gettime(*n+(*p!=0 ? clockoffset : 0), a)));
 
 const char *timestr(int dur, int style)
 {
-    static string buf; buf[0] = 0;
+    static string buf; buf[0] = '\0';
     int tm = dur, ms = 0, ss = 0, mn = 0;
     if(tm > 0)
     {
@@ -72,18 +89,19 @@ const char *timestr(int dur, int style)
     }
     switch(style)
     {
-        case 0: formatstring(buf)("%d.%d", tm, ms/100); break;
-        case 1: formatstring(buf)("%d:%02d.%03d", mn, ss, ms); break;
-        case 2: formatstring(buf)("%d:%02d.%d", mn, ss, ms/100); break;
-        case 3: formatstring(buf)("%d:%02d", mn, ss); break;
+        case 0: formatstring(buf, "%d.%d", tm, ms/100); break;
+        case 1: formatstring(buf, "%d:%02d.%03d", mn, ss, ms); break;
+        case 2: formatstring(buf, "%d:%02d.%d", mn, ss, ms/100); break;
+        case 3: formatstring(buf, "%d:%02d", mn, ss); break;
         case 4:
         {
             if(mn > 0)
             {
-                formatstring(buf)("%dm%ds", mn, ss);
+                if(ss > 0) formatstring(buf, "%dm%ds", mn, ss);
+                else formatstring(buf, "%dm", mn);
                 break;
             }
-            formatstring(buf)("%ds", ss);
+            formatstring(buf, "%ds", ss);
             break;
         }
     }
@@ -122,8 +140,9 @@ ICOMMAND(0, addallow, "ss", (char *name, char *reason), addipinfo(control, ipinf
 ICOMMAND(0, addban, "ss", (char *name, char *reason), addipinfo(control, ipinfo::BAN, name, reason));
 ICOMMAND(0, addmute, "ss", (char *name, char *reason), addipinfo(control, ipinfo::MUTE, name, reason));
 ICOMMAND(0, addlimit, "ss", (char *name, char *reason), addipinfo(control, ipinfo::LIMIT, name, reason));
+ICOMMAND(0, addtrust, "ss", (char *name, char *reason), addipinfo(control, ipinfo::TRUST, name, reason));
 
-const char *ipinfotypes[ipinfo::MAXTYPES] = { "allow", "ban", "mute", "limit" };
+const char *ipinfotypes[ipinfo::MAXTYPES] = { "allow", "ban", "mute", "limit", "trust" };
 char *printipinfo(const ipinfo &info, char *buf)
 {
     static string ipinfobuf = ""; char *str = buf ? buf : (char *)&ipinfobuf;
@@ -195,8 +214,8 @@ void console(int type, const char *s, ...)
 {
     defvformatbigstring(sf, s, s);
     bigstring osf;
-    filterbigstring(osf, sf);
-    if(*logtimeformat) logoutf("%s %s", gettime(currenttime, logtimeformat), osf);
+    filterstring(osf, sf);
+    if(*logtimeformat) logoutf("%s %s", gettime(logtimelocal ? currenttime : clocktime, logtimeformat), osf);
     else logoutf("%s", osf);
 #ifndef STANDALONE
     conline(type, sf, 0);
@@ -250,12 +269,6 @@ VAR(0, serverport, 1, SERVER_PORT, VAR_MAX);
 VAR(0, serverlanport, 0, LAN_PORT, VAR_MAX);
 SVAR(0, serverip, "");
 
-int curtime = 0, totalmillis = 1, lastmillis = 1, timescale = 100, paused = 0, timeerr = 0;
-time_t clocktime = 0, currenttime = 0, clockoffset = 0;
-uint totalsecs = 0;
-const char *load = NULL;
-vector<char *> gameargs;
-
 bool filterword(char *src, const char *list)
 {
     bool filtered = false;
@@ -284,7 +297,7 @@ bool filterword(char *src, const char *list)
 ICOMMAND(0, filterword, "ss", (char *s, char *t),
 {
     char *d = newstring(s);
-    filterstring(d, d, t);
+    filterword(d, t);
     stringret(d);
 });
 
@@ -293,7 +306,7 @@ bool filterstring(char *dst, const char *src, bool newline, bool colour, bool wh
 {
     bool filtered = false;
     size_t n = 0;
-    for(int c = uchar(*src); c && n < len; c = uchar(*++src))
+    for(int c = uchar(*src); c && n <= len; c = uchar(*++src))
     {
         if(newline && (c=='\n' || c=='\r')) c = ' ';
         if(c=='\f')
@@ -323,14 +336,10 @@ bool filterstring(char *dst, const char *src, bool newline, bool colour, bool wh
             dst[n++] = c;
         else filtered = true;
     }
-    if(whitespace && wsstrip && n) while(iscubespace(dst[n-1])) dst[--n] = 0;
-    dst[n < len ? n : len-1] = 0;
+    if(whitespace && wsstrip && n) while(iscubespace(dst[n-1])) dst[--n] = '\0';
+    dst[n <= len ? n : len] = '\0';
     return filtered;
 }
-bool filterbigstring(char *dst, const char *src, bool newline, bool colour, bool whitespace, bool wsstrip, size_t len)
-{
-    return filterstring(dst, src, newline, colour, whitespace, wsstrip, len);
-}
 ICOMMAND(0, filter, "siiiiN", (char *s, int *a, int *b, int *c, int *d, int *numargs),
 {
     size_t len = strlen(s);
@@ -407,6 +416,7 @@ void cleanupserver()
 {
     server::shutdown();
     cleanupserversockets();
+    cleanupmaster();
     irccleanup();
 }
 
@@ -434,7 +444,7 @@ void sendpacket(int n, int chan, ENetPacket *packet, int exclude)
         loopv(clients) if(i != server::peerowner(exclude) && server::allowbroadcast(i)) sendpacket(i, chan, packet, exclude);
         return;
     }
-    switch(clients[n]->type)
+    if(clients.inrange(n)) switch(clients[n]->type)
     {
         case ST_REMOTE:
         {
@@ -470,7 +480,6 @@ void sendf(int cn, int chan, const char *format, ...)
         case 'x':
             exclude = va_arg(args, int);
             break;
-
         case 'v':
         {
             int n = va_arg(args, int);
@@ -478,23 +487,25 @@ void sendf(int cn, int chan, const char *format, ...)
             loopi(n) putint(p, v[i]);
             break;
         }
-
         case 'i':
         {
             int n = isdigit(*format) ? *format++-'0' : 1;
             loopi(n) putint(p, va_arg(args, int));
             break;
         }
-
+        case 'u':
+        {
+            int n = isdigit(*format) ? *format++-'0' : 1;
+            loopi(n) putuint(p, va_arg(args, uint));
+            break;
+        }
         case 'f':
         {
             int n = isdigit(*format) ? *format++-'0' : 1;
             loopi(n) putfloat(p, (float)va_arg(args, double));
             break;
         }
-
         case 's': sendstring(va_arg(args, const char *), p); break;
-
         case 'm':
         {
             int n = va_arg(args, int);
@@ -516,30 +527,56 @@ void sendfile(int cn, int chan, stream *file, const char *format, ...)
     }
     else if(!clients.inrange(cn)) return;
 
-    int len = (int)min(file->size(), stream::offset(INT_MAX));
-    if(len <= 0 || len > 16<<20) return;
+    int len = file ? (int)min(file->size(), stream::offset(INT_MAX)) : 0;
+    if(len > 16<<20) return;
 
     packetbuf p(MAXTRANS+len, ENET_PACKET_FLAG_RELIABLE);
     va_list args;
     va_start(args, format);
     while(*format) switch(*format++)
     {
+        case 'l': putint(p, len); break;
+        case 'v':
+        {
+            int n = va_arg(args, int);
+            int *v = va_arg(args, int *);
+            loopi(n) putint(p, v[i]);
+            break;
+        }
         case 'i':
         {
             int n = isdigit(*format) ? *format++-'0' : 1;
             loopi(n) putint(p, va_arg(args, int));
             break;
         }
+        case 'u':
+        {
+            int n = isdigit(*format) ? *format++-'0' : 1;
+            loopi(n) putuint(p, va_arg(args, uint));
+            break;
+        }
+        case 'f':
+        {
+            int n = isdigit(*format) ? *format++-'0' : 1;
+            loopi(n) putfloat(p, (float)va_arg(args, double));
+            break;
+        }
         case 's': sendstring(va_arg(args, const char *), p); break;
-        case 'l': putint(p, len); break;
+        case 'm':
+        {
+            int n = va_arg(args, int);
+            p.put(va_arg(args, uchar *), n);
+            break;
+        }
     }
     va_end(args);
-
-    file->seek(0, SEEK_SET);
-    file->read(p.subbuf(len).buf, len);
-
+    if(file && len > 0)
+    {
+        file->seek(0, SEEK_SET);
+        file->read(p.subbuf(len).buf, len);
+    }
     ENetPacket *packet = p.finalize();
-    if(cn >= 0) sendpacket(cn, chan, packet, -1);
+    if(cn >= 0) sendpacket(cn, chan, packet);
 #ifndef STANDALONE
     else sendclientpacket(packet, chan);
 #endif
@@ -692,14 +729,8 @@ bool connectedmaster() { return mastersock != ENET_SOCKET_NULL; }
 
 bool requestmaster(const char *req)
 {
-    // RE master server need an init message
-    //if(mastersock == ENET_SOCKET_NULL)
-    //{
-    //    mastersock = connectmaster();
-        if(mastersock == ENET_SOCKET_NULL) return false;
-    //}
+    if(mastersock == ENET_SOCKET_NULL) return false;
     if(masterout.length() >= 4096) return false;
-
     masterout.put(req, strlen(req));
     return true;
 }
@@ -895,19 +926,23 @@ void serverslice(uint timeout)  // main server update, called from main loop in
                     if(enet_address_get_host(&c.peer->address, c.hostname, sizeof(c.hostname)) >= 0)
                     {
                         ENetAddress address;
-                        string hostname;
-                        if(enet_address_set_host(&address, c.hostname) < 0 || enet_address_get_host_ip(&address, hostname, sizeof(hostname)) < 0 || strcmp(hostname, c.hostname))
+                        string ip;
+                        conoutf("checking reverse lookup of %s (%s)", c.hostip, c.hostname);
+                        if(enet_address_set_host(&address, c.hostname) < 0 || enet_address_get_host_ip(&address, ip, sizeof(ip)) < 0 || strcmp(ip, c.hostip))
+                        {
+                            conoutf("reverse lookup of %s (%s) failed (%s), using ip address", c.hostip, c.hostname, ip);
                             copystring(c.hostname, c.hostip);
+                        }
                     }
-                    else copystring(c.hostname, c.hostip);
+                    else
+                    {
+                        conoutf("reverse lookup of %s failed, using ip address", c.hostip);
+                        copystring(c.hostname, c.hostip);
+                    }
+                    int reason = server::clientconnect(c.num, c.peer->address.host);
+                    if(reason) disconnect_client(c.num, reason);
                 }
-                else
-                {
-                    copystring(c.hostname, "unknown");
-                    copystring(c.hostip, "0.0.0.0");
-                }
-                int reason = server::clientconnect(c.num, c.peer->address.host);
-                if(reason) disconnect_client(c.num, reason);
+                else disconnect_client(c.num, DISC_HOSTFAIL);
                 break;
             }
             case ENET_EVENT_TYPE_RECEIVE:
@@ -985,11 +1020,16 @@ int updatetimer(bool limit)
     lastmillis += curtime;
     totalmillis = millis;
     static int lastsec = 0;
-    if(totalmillis - lastsec >= 1000)
+    if(totalmillis-lastsec >= 1000)
     {
-        int cursecs = (totalmillis - lastsec) / 1000;
+        int cursecs = (totalmillis-lastsec)/1000;
         totalsecs += cursecs;
-        lastsec += cursecs * 1000;
+        lastsec += cursecs*1000;
+        ifserver(maxruntime && !shutdownwait && int(totalsecs) >= maxruntime)
+        {
+            server::srvoutf(-3, "\fymax run time reached (\fs\fc%s\fS), waiting for server to empty", timestr(maxruntime*1000, 4));
+            shutdownwait = totalmillis;
+        }
     }
     return elapsed;
 }
@@ -1098,8 +1138,8 @@ static void setupconsole()
     SetConsoleCtrlHandler(consolehandler, TRUE);
     conwindow = GetConsoleWindow();
     SetConsoleTitle(apptip);
-	//SendMessage(conwindow, WM_SETICON, ICON_SMALL, (LPARAM)appicon);
-	SendMessage(conwindow, WM_SETICON, ICON_BIG, (LPARAM)appicon);
+    //SendMessage(conwindow, WM_SETICON, ICON_SMALL, (LPARAM)appicon);
+    SendMessage(conwindow, WM_SETICON, ICON_BIG, (LPARAM)appicon);
     outhandle = GetStdHandle(STD_OUTPUT_HANDLE);
     CONSOLE_SCREEN_BUFFER_INFO coninfo;
     GetConsoleScreenBufferInfo(outhandle, &coninfo);
@@ -1201,7 +1241,7 @@ static void setupwindow(const char *title)
     atexit(cleanupwindow);
 
     if(!setupsystemtray(WM_APP)) fatal("failed adding to system tray");
-    conoutf("identity: v%s-%s%d %s (%s) [0x%x]", VERSION_STRING, versionplatname, versionarch, versionisserver ? "server" : "client", VERSION_RELEASE, versioncrc);
+    conoutf("identity: %s@%s v%s-%s%d-%s %s (%s) [0x%.8x]", systemuser, systemhost, versionstring, versionplatname, versionarch, versionbranch, versionisserver ? "server" : "client", versionrelease, versioncrc);
 }
 
 static char *parsecommandline(const char *src, vector<char *> &args)
@@ -1212,12 +1252,12 @@ static char *parsecommandline(const char *src, vector<char *> &args)
         while(isspace(*src)) src++;
         if(!*src) break;
         args.add(dst);
- 		for(bool quoted = false; *src && (quoted || !isspace(*src)); src++)
+         for(bool quoted = false; *src && (quoted || !isspace(*src)); src++)
         {
             if(*src != '"') *dst++ = *src;
-			else if(dst > buf && src[-1] == '\\') dst[-1] = '"';
-			else quoted = !quoted;
-		}
+            else if(dst > buf && src[-1] == '\\') dst[-1] = '"';
+            else quoted = !quoted;
+        }
         *dst++ = '\0';
     }
     args.add(NULL);
@@ -1235,7 +1275,7 @@ int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
     int status = standalonemain(args.length()-1, args.getbuf());
     #define main standalonemain
 #else
-    SDL_SetModuleHandle(hInst);
+    SDL_SetMainReady();
     int status = SDL_main(args.length()-1, args.getbuf());
 #endif
     delete[] buf;
@@ -1271,7 +1311,7 @@ void logoutfv(const char *fmt, va_list args)
 void serverloop()
 {
 #ifdef WIN32
-    defformatstring(cap)("%s server", VERSION_NAME);
+    defformatstring(cap, "%s server", versionname);
     setupwindow(cap);
     SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
 #endif
@@ -1282,9 +1322,7 @@ void serverloop()
         //lastmillis = totalmillis = (int)enet_time_get();
         //curtime = lastmillis-_lastmillis;
         updatetimer(false);
-#ifdef STANDALONE
         checkmaster();
-#endif
         serverslice(5);
         ircslice();
 #ifdef WIN32
@@ -1389,9 +1427,7 @@ void setupserver()
 {
     server::changemap(load && *load ? load : NULL);
     if(!servertype) return;
-#ifdef STANDALONE
     setupmaster();
-#endif
     conoutf("loading server (%s:%d)..", *serverip ? serverip : "*", serverport);
     if(setupserversockets() && verbose) conoutf("game server started");
 #ifndef STANDALONE
@@ -1401,7 +1437,7 @@ void setupserver()
 
 void initgame()
 {
-    conoutf("identity: v%s-%s%d %s (%s) [0x%x]", VERSION_STRING, versionplatname, versionarch, versionisserver ? "server" : "client", VERSION_RELEASE, versioncrc);
+    conoutf("identity: %s@%s v%s-%s%d-%s %s (%s) [0x%.8x]", systemuser, systemhost, versionstring, versionplatname, versionarch, versionbranch, versionisserver ? "server" : "client", versionrelease, versioncrc);
     server::start();
     loopv(gameargs)
     {
@@ -1450,9 +1486,10 @@ bool serveroption(char *opt)
                 case 's': setvar("servertype", atoi(opt+3)); return true;
                 case 'p': setvar("serverport", atoi(opt+3)); return true;
                 case 'a': setvar("servermasterport", atoi(opt+3)); return true;
+                case 'g': setvar("logtimelocal", atoi(opt+3)); return true;
+                default: return false;
             }
         }
-#ifdef STANDALONE
         case 'm':
         {
             switch(opt[2])
@@ -1464,7 +1501,6 @@ bool serveroption(char *opt)
             }
             return false;
         }
-#endif
         default: return false;
     }
     return false;
@@ -1475,12 +1511,12 @@ bool findoctadir(const char *name, bool fallback)
     string s = "";
     copystring(s, name);
     path(s);
-    defformatstring(octadefs)("%s/data/default_map_settings.cfg", s);
+    defformatstring(octadefs, "%s/data/default_map_settings.cfg", s);
     if(fileexists(findfile(octadefs, "r"), "r"))
     {
         conoutf("\fwfound octa directory: %s", s);
-        defformatstring(octadata)("%s/data", s);
-        defformatstring(octapaks)("%s/packages", s);
+        defformatstring(octadata, "%s/data", s);
+        defformatstring(octapaks, "%s/packages", s);
         addpackagedir(s, PACKAGEDIR_OCTA);
         addpackagedir(octadata, PACKAGEDIR_OCTA);
         addpackagedir(octapaks, PACKAGEDIR_OCTA);
@@ -1517,7 +1553,7 @@ void trytofindocta(bool fallback)
         string dir = "";
         if(SHGetFolderPath(NULL, CSIDL_PROGRAM_FILESX86, NULL, 0, dir) == S_OK)
         {
-            defformatstring(s)("%s\\Sauerbraten", dir);
+            defformatstring(s, "%s\\Sauerbraten", dir);
             if(findoctadir(s, true)) return;
         }
 #elif defined(__APPLE__)
@@ -1541,33 +1577,24 @@ void trytofindocta(bool fallback)
 
 void setlocations(bool wanthome)
 {
-#if defined(__APPLE__)
-    extern const char *mac_resourcedir();
-    const char *dir = mac_resourcedir(); // ./blah.app/Contents/Resources
-    if(dir && *dir && !chdir(dir))
-        conoutf("attempting to use resources in: %s", dir);
-#endif
-    loopirev(3) if(!fileexists(findfile("config/version.cfg", "r"), "r"))
+    int backstep = 3;
+    loopirev(backstep) if(!fileexists(findfile("config/version.cfg", "r"), "r"))
     { // standalone solution to this is: pebkac
         if(!i || chdir("..") < 0) fatal("could not find config directory");
     }
     if(!execfile("config/version.cfg", false, EXEC_VERSION|EXEC_BUILTIN)) fatal("cannot exec 'config/version.cfg'");
     // pseudo directory with game content
-    const char *dir = getenv("GAME_DATA");
+    const char *dir = getenv(sup_var("DATADIR"));
     if(dir && *dir) addpackagedir(dir);
     else addpackagedir("data");
-#ifndef STANDALONE
-    if(!fileexists(findfile("textures/logo.png", "r"), "r")) fatal("could not find game data");
-#endif
-    //defformatstring(gamedata)("game/%s", server::gameid());
-    //addpackagedir(gamedata);
+    if(!fileexists(findfile("maps/readme.txt", "r"), "r")) fatal("could not find game data");
     if(wanthome)
     {
 #if defined(WIN32)
         string dir = "";
         if(SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, dir) == S_OK)
         {
-            defformatstring(s)("%s\\My Games\\%s", dir, VERSION_NAME);
+            defformatstring(s, "%s\\My Games\\%s", dir, versionname);
             sethomedir(s);
         }
 #elif defined(__APPLE__)
@@ -1575,14 +1602,14 @@ void setlocations(bool wanthome)
         const char *dir = mac_personaldir(); // typically  /Users/<name>/Application Support/
         if(dir && *dir)
         {
-            defformatstring(s)("%s/%s", dir, VERSION_UNAME);
+            defformatstring(s, "%s/%s", dir, versionname);
             sethomedir(s);
         }
 #else
         const char *dir = getenv("HOME");
         if(dir && *dir)
         {
-            defformatstring(s)("%s/.%s", dir, VERSION_UNAME);
+            defformatstring(s, "%s/.%s", dir, versionuname);
             sethomedir(s);
         }
 #endif
@@ -1590,14 +1617,19 @@ void setlocations(bool wanthome)
     }
 }
 
+#ifndef STANDALONE
+VAR(IDF_PERSIST, noconfigfile, 0, 0, 1);
+#endif // STANDALONE
+
 void writecfg()
 {
 #ifndef STANDALONE
+    if(noconfigfile) return;
     stream *f = openutf8file("config.cfg", "w");
     if(!f) return;
     vector<ident *> ids;
     enumerate(idents, ident, id, ids.add(&id));
-    ids.sort(ident::compare);
+    ids.sortname();
     bool found = false;
     loopv(ids)
     {
@@ -1631,7 +1663,6 @@ void writecfg()
     delete f;
 #endif
 }
-
 ICOMMAND(0, writecfg, "", (void), if(!(identflags&IDF_WORLD)) writecfg());
 
 void rehash(bool reload)
@@ -1642,17 +1673,18 @@ void rehash(bool reload)
         writecfg();
     }
     reloadserver();
-#ifdef STANDALONE
     reloadmaster();
+#ifdef STANDALONE
     execfile("servinit.cfg", false, EXEC_VERSION);
 #else
-    execfile("localinit.cfg", false, EXEC_VERSION);
+    if(servertype >= 3) execfile("servinit.cfg", false, EXEC_VERSION);
+    else execfile("localinit.cfg", false, EXEC_VERSION);
     initing = INIT_DEFAULTS;
     execfile("config/defaults.cfg");
     initing = INIT_LOAD;
     interactive = true;
     execfile("servers.cfg", false);
-    execfile("config.cfg", false);
+    if(!noconfigfile) execfile("config.cfg", false);
     execfile("autoexec.cfg", false);
     interactive = false;
     initing = NOT_INITING;
@@ -1662,18 +1694,84 @@ void rehash(bool reload)
 }
 ICOMMAND(0, rehash, "i", (int *nosave), if(!(identflags&IDF_WORLD)) rehash(*nosave ? false : true));
 
-void setcrc(const char *bin)
+#ifndef WIN32
+#include <pwd.h>
+#endif
+void setverinfo(const char *bin)
 {
-    if(!bin || !*bin) return;
-    size_t len = 0;
-    char *buf = loadfile(bin, &len, false);
-    if(!buf) return;
-    versioncrc = crc32(0, (const Bytef *)buf, len);
-    delete[] buf;
+    string buf;
+    setvar("versioncrc", crcfile(bin));
+    const char *vbranch = getenv(sup_var("BRANCH"));
+    setsvar("versionbranch", vbranch && *vbranch ? vbranch : "none");
+#ifdef WIN32
+    const char *suser = getenv("USERNAME");
+#else
+    const char *suser = getenv("USER");
+    if(!suser || !*suser)
+    { // only fall back to this if the variable isn't exported
+        passwd *p = getpwuid(geteuid());
+        if(p)
+        {
+            copystring(buf, p->pw_name);
+            suser = buf;
+        }
+    }
+#endif
+    setsvar("systemuser", suser && *suser ? suser : "none");
+#ifdef WIN32
+    const char *shost = getenv("COMPUTERNAME");
+#else
+    const char *shost = getenv("HOSTNAME");
+#endif
+    if(!shost || !*shost)
+    { // only fall back to this if the variable isn't exported
+        if(!gethostname(buf, sizeof(string)-1) && *buf) shost = buf;
+    }
+    setsvar("systemhost", shost && *shost ? shost : "unknown");
+}
+
+volatile bool fatalsig = false;
+void fatalsignal(int signum)
+{
+    if(!fatalsig)
+    {
+        fatalsig = true;
+        const char *str = "Error: Fatal signal %d (Unknown Error)";
+        switch(signum)
+        {
+            case SIGINT: str = "Exit signal %d (Interrupt)"; break;
+            case SIGILL: str = "Fatal signal %d (Illegal Instruction)"; break;
+            case SIGABRT: str = "Fatal signal %d (Aborted)"; break;
+            case SIGFPE: str = "Fatal signal %d (Floating-point Exception)"; break;
+            case SIGSEGV: str = "Fatal signal %d (Segmentation Violation)"; break;
+            case SIGTERM: str = "Exit signal %d (Terminated)"; break;
+#if !defined(WIN32) && !defined(__APPLE__)
+            case SIGQUIT: str = "Exit signal %d (Quit)"; break;
+            case SIGKILL: str = "Fatal signal %d (Killed)"; break;
+            case SIGPIPE: str = "Fatal signal %d (Broken Pipe)"; break;
+            case SIGALRM: str = "Fatal signal %d (Alarm)"; break;
+#endif
+            default: break;
+        }
+        fatal(str, signum);
+    }
+}
+
+void reloadsignal(int signum)
+{
+    rehash(true);
+}
+
+void shutdownsignal(int signum)
+{
+#ifndef STANDALONE
+    if(servertype < 3) fatalsignal(signum);
+#endif
+    server::srvoutf(-3, "\fyshutdown signal received, waiting for server to empty");
+    shutdownwait = totalmillis;
 }
 
 #ifdef STANDALONE
-#include <signal.h>
 volatile int errors = 0;
 void fatal(const char *s, ...)    // failure exit
 {
@@ -1687,12 +1785,9 @@ void fatal(const char *s, ...)    // failure exit
         if(errors <= 1) // avoid recursion
         {
             cleanupserver();
-#ifdef STANDALONE
-            cleanupmaster();
-#endif
             enet_deinitialize();
 #ifdef WIN32
-            defformatstring(cap)("%s: Error", VERSION_NAME);
+            defformatstring(cap, "%s: Error", versionname);
             MessageBox(NULL, msg, cap, MB_OK|MB_SYSTEMMODAL);
 #endif
         }
@@ -1700,21 +1795,6 @@ void fatal(const char *s, ...)    // failure exit
     exit(EXIT_FAILURE);
 }
 
-volatile bool fatalsig = false;
-void fatalsignal(int signum)
-{
-    if(!fatalsig)
-    {
-        fatalsig = true;
-        fatal("Received fatal signal %d", signum);
-    }
-}
-
-void reloadsignal(int signum)
-{
-    rehash(true);
-}
-
 int main(int argc, char **argv)
 {
     currenttime = time(NULL); // initialise
@@ -1723,7 +1803,7 @@ int main(int argc, char **argv)
 
     setlogfile(NULL);
     setlocations(true);
-    setcrc(argv[0]);
+    setverinfo(argv[0]);
 
     char *initscript = NULL;
     for(int i = 1; i<argc; i++)
@@ -1737,13 +1817,14 @@ int main(int argc, char **argv)
     }
 
     if(enet_initialize()<0) fatal("Unable to initialise network module");
+    atexit(enet_deinitialize);
 
     signal(SIGINT, fatalsignal);
     signal(SIGILL, fatalsignal);
     signal(SIGABRT, fatalsignal);
     signal(SIGFPE, fatalsignal);
     signal(SIGSEGV, fatalsignal);
-    signal(SIGTERM, fatalsignal);
+    signal(SIGTERM, shutdownsignal);
     signal(SIGINT, fatalsignal);
 #if !defined(WIN32) && !defined(__APPLE__)
     signal(SIGHUP, reloadsignal);
@@ -1758,7 +1839,7 @@ int main(int argc, char **argv)
     trytofindocta();
     if(initscript) execute(initscript);
     serverloop();
+    cleanupserver();
     return 0;
 }
 #endif
-
