@@ -34,20 +34,21 @@ void sound::reset()
     slotnum = chan = -1;
     if(hook) *hook = -1;
     hook = NULL;
+    buffer.shrink(0);
 }
 
-hashtable<const char *, soundsample> soundsamples;
+hashnameset<soundsample> soundsamples;
 vector<soundslot> gamesounds, mapsounds;
 vector<sound> sounds;
 
-bool nosound = true, changedvol = false;
+bool nosound = true, changedvol = false, canmusic = false;
 Mix_Music *music = NULL;
 SDL_RWops *musicrw = NULL;
 stream *musicstream = NULL;
 char *musicfile = NULL, *musicdonecmd = NULL;
 int musictime = -1, musicdonetime = -1;
 
-VARF(IDF_PERSIST, mastervol, 0, 255, 255, changedvol = true);
+VARF(IDF_PERSIST, mastervol, 0, 255, 255, changedvol = true; if(!music && musicvol > 0 && mastervol > 0) smartmusic(true));
 VAR(IDF_PERSIST, soundvol, 0, 255, 255);
 VARF(0, soundmono, 0, 0, 1, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
 VARF(0, soundmixchans, 16, 32, 1024, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
@@ -61,7 +62,7 @@ FVAR(IDF_PERSIST, soundenvvol, 0, 1, FVAR_MAX);
 FVAR(IDF_PERSIST, soundenvscale, 0, 1, FVAR_MAX);
 VAR(IDF_PERSIST, soundcull, 0, 1, 1);
 
-VARF(IDF_PERSIST, musicvol, 0, 64, 255, changedvol = true);
+VARF(IDF_PERSIST, musicvol, 0, 64, 255, changedvol = true; if(!music && musicvol > 0) smartmusic(true));
 VAR(IDF_PERSIST, musicfadein, 0, 1000, VAR_MAX);
 VAR(IDF_PERSIST, musicfadeout, 0, 2500, VAR_MAX);
 SVAR(0, titlemusic, "sounds/theme");
@@ -178,8 +179,8 @@ void getcursounds(int idx, int prop)
             case 8: intret(sounds[idx].ends); break;
             case 9: intret(sounds[idx].slotnum); break;
             case 10: intret(sounds[idx].chan); break;
-            case 11: defformatstring(pos)("%.f %.f %.f", sounds[idx].pos.x, sounds[idx].pos.y, sounds[idx].pos.z); result(pos); break;
-            case 12: defformatstring(oldpos)("%.f %.f %.f", sounds[idx].oldpos.x, sounds[idx].oldpos.y, sounds[idx].oldpos.z); result(oldpos); break;
+            case 11: defformatstring(pos, "%.f %.f %.f", sounds[idx].pos.x, sounds[idx].pos.y, sounds[idx].pos.z); result(pos); break;
+            case 12: defformatstring(oldpos, "%.f %.f %.f", sounds[idx].oldpos.x, sounds[idx].oldpos.y, sounds[idx].oldpos.z); result(oldpos); break;
             case 13: intret(sounds[idx].valid() ? 1 : 0); break;
             case 14: intret(sounds[idx].playing() ? 1 : 0); break;
             case 15: intret(sounds[idx].flags&SND_MAP ? 1 : 0); break;
@@ -200,7 +201,7 @@ Mix_Music *loadmusic(const char *name)
         if(!musicrw) musicrw = musicstream->rwops();
         if(!musicrw) DELETEP(musicstream);
     }
-    if(musicrw) music = Mix_LoadMUS_RW(musicrw);
+    if(musicrw) music = Mix_LoadMUSType_RW(musicrw, MUS_NONE, 0);
     else music = Mix_LoadMUS(findfile(name, "rb"));
     if(!music)
     {
@@ -225,7 +226,7 @@ bool playmusic(const char *name, const char *cmd)
         {
             loopk(sizeof(exts)/sizeof(exts[0]))
             {
-                formatstring(buf)("%s%s%s", dirs[i], name, exts[k]);
+                formatstring(buf, "%s%s%s", dirs[i], name, exts[k]);
                 if(loadmusic(buf))
                 {
                     DELETEA(musicfile);
@@ -270,9 +271,10 @@ bool playingmusic(bool check)
     return false;
 }
 
-void smartmusic(bool cond, bool autooff)
+void smartmusic(bool cond, bool init)
 {
-    if(nosound || !mastervol || !musicvol || (!cond && Mix_PlayingMusic()) || !*titlemusic) return;
+    if(init) canmusic = true;
+    if(!canmusic || nosound || !mastervol || !musicvol || (!cond && Mix_PlayingMusic()) || !*titlemusic) return;
     if(!playingmusic() || (cond && strcmp(musicfile, titlemusic))) playmusic(titlemusic);
     else
     {
@@ -280,7 +282,7 @@ void smartmusic(bool cond, bool autooff)
         changedvol = true;
     }
 }
-ICOMMAND(0, smartmusic, "ii", (int *a, int *b), smartmusic(*a, *b));
+ICOMMAND(0, smartmusic, "i", (int *a), smartmusic(*a));
 
 int findsound(const char *name, int vol, vector<soundslot> &soundset)
 {
@@ -347,7 +349,7 @@ int addsound(const char *name, int vol, int maxrad, int minrad, int value, vecto
             { \
                 loopk(sizeof(exts)/sizeof(exts[0])) \
                 { \
-                    formatstring(buf)("%s%s%s", dirs[i], sample->name, exts[k]); \
+                    formatstring(buf, "%s%s%s", dirs[i], sample->name, exts[k]); \
                     if((sample->sound = loadwav(buf)) != NULL) found = true; \
                     if(found) break; \
                 } \
@@ -358,7 +360,7 @@ int addsound(const char *name, int vol, int maxrad, int minrad, int value, vecto
     string sam;
     loopi(value > 1 ? 2 : 1)
     {
-        if(value > 1 && !i) formatstring(sam)("%s1", name);
+        if(value > 1 && !i) formatstring(sam, "%s1", name);
         else copystring(sam, name);
         loadsound(sam);
         if(!sample->sound)
@@ -375,7 +377,7 @@ int addsound(const char *name, int vol, int maxrad, int minrad, int value, vecto
     slot.samples.add(sample);
     if(value > 1) loopi(value-1)
     {
-        formatstring(sam)("%s%d", name, i+2);
+        formatstring(sam, "%s%d", name, i+2);
         loadsound(sam);
         if(!sample->sound) conoutf("\frfailed to load sample: %s", sam);
         else slot.samples.add(sample);
@@ -419,7 +421,7 @@ void updatesound(int chan)
 {
     sound &s = sounds[chan];
     bool waiting = (!(s.flags&SND_NODELAY) && Mix_Paused(chan));
-    if((s.flags&SND_NOCULL) || !soundcull || s.curvol > 0 || s.pos.dist(camera1->o) <= 0)
+    if((s.flags&SND_NOCULL) || (s.flags&SND_BUFFER) || !soundcull || s.curvol > 0 || s.pos.dist(camera1->o) <= 0)
     {
         if(waiting)
         { // delay the sound based on average physical constants
@@ -435,12 +437,7 @@ void updatesound(int chan)
         if(!waiting)
         {
             Mix_Volume(chan, s.curvol);
-            if(!soundmono)
-            {
-                SDL_LockAudio(); // workaround for race condition in inside Mix_SetPanning
-                Mix_SetPanning(chan, 255-s.curpan, s.curpan);
-                SDL_UnlockAudio();
-            }
+            if(!soundmono) Mix_SetPanning(chan, 255-s.curpan, s.curpan);
         }
     }
     else
@@ -465,8 +462,62 @@ void updatesounds()
             calcvol(s.flags, s.vol, s.slot->vol, s.maxrad, s.minrad, s.pos, &s.curvol, &s.curpan, liquid || isliquid(s.material&MATF_VOLUME));
             s.oldpos = s.pos;
             updatesound(i);
+            continue;
         }
-        else removesound(i);
+        vector<soundslot> &soundset = s.flags&SND_MAP ? mapsounds : gamesounds;
+        while(!s.buffer.empty() && (!soundset.inrange(s.buffer[0]) || soundset[s.buffer[0]].samples.empty() || !soundset[s.buffer[0]].vol)) s.buffer.remove(0);
+        if(!s.buffer.empty())
+        {
+            int n = s.buffer[0], chan = -1;
+            Mix_HaltChannel(i);
+            s.buffer.remove(0);
+            bool nocull = s.flags&SND_NOCULL || s.pos.dist(camera1->o) <= 0;
+            soundslot *slot = &soundset[n];
+            soundsample *sample = slot->samples[rnd(slot->samples.length())];
+            if((chan = Mix_PlayChannel(i, sample->sound, s.flags&SND_LOOP ? -1 : 0)) < 0)
+            {
+                int lowest = -1;
+                loopv(sounds) if(sounds[i].chan >= 0 && (!(sounds[i].flags&SND_MAP) || s.flags&SND_MAP) && sounds[i].curvol < s.curvol && (lowest < 0 || sounds[i].curvol < sounds[lowest].curvol) && (nocull || (!(sounds[i].flags&SND_NOCULL) && sounds[i].pos.dist(camera1->o) > 0)))
+                    lowest = i;
+                if(sounds.inrange(lowest))
+                {
+                    if(verbose >= 4) conoutf("culled channel %d (%d)", lowest, sounds[lowest].curvol);
+                    removesound(lowest);
+                    chan = Mix_PlayChannel(-1, sample->sound, s.flags&SND_LOOP ? -1 : 0);
+                }
+            }
+            if(chan >= 0)
+            {
+                while(chan >= sounds.length()) sounds.add();
+                sound &t = sounds[chan];
+                if(chan == s.chan)
+                {
+                    i--; // repeat with new sound
+                    continue;
+                }
+                else
+                {
+                    t.slot = slot;
+                    t.vol = s.vol;
+                    t.maxrad = s.maxrad;
+                    t.minrad = s.minrad;
+                    t.material = s.material;
+                    t.flags = s.flags;
+                    t.millis = s.millis;
+                    t.ends = s.ends;
+                    t.slotnum = s.slotnum;
+                    t.owner = s.owner;
+                    t.pos = t.oldpos = s.pos;
+                    t.curvol = s.curvol;
+                    t.curpan = s.curpan;
+                    t.chan = chan;
+                    t.hook = s.hook;
+                    loopvj(s.buffer) t.buffer.add(s.buffer[j]);
+                    updatesound(chan);
+                }
+            }
+        }
+        removesound(i);
     }
     if(music || Mix_PlayingMusic())
     {
@@ -487,6 +538,11 @@ int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad,
 
     if(soundset.inrange(n))
     {
+        if(hook && issound(*hook) && flags&SND_BUFFER)
+        {
+            sounds[*hook].buffer.add(n);
+            return *hook;
+        }
         if(soundset[n].samples.empty() || !soundset[n].vol)
         {
             if(oldhook && issound(*oldhook)) removesound(*oldhook);
@@ -513,7 +569,7 @@ int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad,
             else
             {
                 oldhook = NULL;
-                soundsample *sample = soundset[n].samples[rnd(soundset[n].samples.length())];
+                soundsample *sample = slot->samples[rnd(slot->samples.length())];
                 if((chan = Mix_PlayChannel(-1, sample->sound, flags&SND_LOOP ? -1 : 0)) < 0)
                 {
                     int lowest = -1;
@@ -560,7 +616,7 @@ int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad,
                 return chan;
             }
             else if(verbose >= 2)
-                conoutf("\frcannot play sound %d (%s): %s", n, soundset[n].name, Mix_GetError());
+                conoutf("\frcannot play sound %d (%s): %s", n, slot->name, Mix_GetError());
         }
         else if(verbose >= 4) conoutf("culled sound %d (%d)", n, cvol);
     }
@@ -569,11 +625,11 @@ int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad,
     return -1;
 }
 
-void sound(int *n, int *vol)
+void sound(int *n, int *vol, int *flags)
 {
-    intret(playsound(*n, camera1->o, camera1, SND_FORCED, *vol ? *vol : -1));
+    intret(playsound(*n, camera1->o, camera1, *flags >= 0 ? *flags : SND_FORCED, *vol ? *vol : -1));
 }
-COMMAND(0, sound, "ii");
+COMMAND(0, sound, "iib");
 
 void removetrackedsounds(physent *d)
 {
@@ -584,12 +640,6 @@ void removetrackedsounds(physent *d)
 
 void resetsound()
 {
-    const SDL_version *v = Mix_Linked_Version();
-    if(SDL_VERSIONNUM(v->major, v->minor, v->patch) <= SDL_VERSIONNUM(1, 2, 8))
-    {
-        conoutf("\frsound reset not available in-game due to SDL_mixer-1.2.8 bug, please restart for changes to take effect.");
-        return;
-    }
     clearchanges(CHANGE_SOUND);
     if(!nosound)
     {
@@ -640,7 +690,7 @@ COMMAND(0, resetsound, "");
 
 #include <unistd.h>
 
-#ifdef _POSIX_SHARED_MEMORY_OBJECTS
+#if _POSIX_SHARED_MEMORY_OBJECTS > 0
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
@@ -650,7 +700,7 @@ COMMAND(0, resetsound, "");
 
 #endif
 
-#if defined(WIN32) || defined(_POSIX_SHARED_MEMORY_OBJECTS)
+#if defined(WIN32) || _POSIX_SHARED_MEMORY_OBJECTS > 0
 struct MumbleInfo
 {
     int version, timestamp;
@@ -663,7 +713,7 @@ struct MumbleInfo
 static HANDLE mumblelink = NULL;
 static MumbleInfo *mumbleinfo = NULL;
 #define VALID_MUMBLELINK (mumblelink && mumbleinfo)
-#elif defined(_POSIX_SHARED_MEMORY_OBJECTS)
+#elif _POSIX_SHARED_MEMORY_OBJECTS > 0
 static int mumblelink = -1;
 static MumbleInfo *mumbleinfo = (MumbleInfo *)-1;
 #define VALID_MUMBLELINK (mumblelink >= 0 && mumbleinfo != (MumbleInfo *)-1)
@@ -686,20 +736,20 @@ void initmumble()
         if(mumblelink)
         {
             mumbleinfo = (MumbleInfo *)MapViewOfFile(mumblelink, FILE_MAP_ALL_ACCESS, 0, 0, sizeof(MumbleInfo));
-            if(mumbleinfo) wcsncpy(mumbleinfo->name, (const wchar_t *)VERSION_UNAME, 256);
+            if(mumbleinfo) wcsncpy(mumbleinfo->name, (const wchar_t *)versionuname, 256);
         }
-    #elif defined(_POSIX_SHARED_MEMORY_OBJECTS)
-        defformatstring(shmname)("/MumbleLink.%d", getuid());
+    #elif _POSIX_SHARED_MEMORY_OBJECTS > 0
+        defformatstring(shmname, "/MumbleLink.%d", getuid());
         mumblelink = shm_open(shmname, O_RDWR, 0);
         if(mumblelink >= 0)
         {
             mumbleinfo = (MumbleInfo *)mmap(NULL, sizeof(MumbleInfo), PROT_READ|PROT_WRITE, MAP_SHARED, mumblelink, 0);
-            if(mumbleinfo != (MumbleInfo *)-1) wcsncpy(mumbleinfo->name, (const wchar_t *)VERSION_UNAME, 256);
+            if(mumbleinfo != (MumbleInfo *)-1) wcsncpy(mumbleinfo->name, (const wchar_t *)versionuname, 256);
         }
     #endif
     if(!VALID_MUMBLELINK) closemumble();
 #else
-    conoutf(CON_ERROR, "Mumble positional audio is not available on this platform.");
+    conoutft(CON_MESG, "Mumble positional audio is not available on this platform.");
 #endif
 }
 
@@ -708,7 +758,7 @@ void closemumble()
 #ifdef WIN32
     if(mumbleinfo) { UnmapViewOfFile(mumbleinfo); mumbleinfo = NULL; }
     if(mumblelink) { CloseHandle(mumblelink); mumblelink = NULL; }
-#elif defined(_POSIX_SHARED_MEMORY_OBJECTS)
+#elif _POSIX_SHARED_MEMORY_OBJECTS > 0
     if(mumbleinfo != (MumbleInfo *)-1) { munmap(mumbleinfo, sizeof(MumbleInfo)); mumbleinfo = (MumbleInfo *)-1; }
     if(mumblelink >= 0) { close(mumblelink); mumblelink = -1; }
 #endif
