@@ -1,23 +1,25 @@
 #ifdef GAMESERVER
 struct duelservmode : servmode
 {
-    int duelround, dueltime, duelcheck, dueldeath, duelwinner, duelwins;
+    int duelround, dueltime, duelcheck, dueldeath, duelwinner, duelwins, duelaffin;
     bool waitforhumans;
-    vector<clientinfo *> duelqueue, allowed, playing, respawns;
+    vector<clientinfo *> duelqueue, allowed, playing, restricted;
 
     duelservmode() {}
 
+    #define DSGS(x) DSG(gamemode, mutators, x)
+
     void shrink()
     {
         allowed.shrink(0);
         playing.shrink(0);
-        respawns.shrink(0);
     }
 
-    void doreset()
+    void doreset(bool start)
     {
-        dueltime = G(duelcooloff);
+        dueltime = start ? 1 : DSGS(cooloff);
         duelcheck = dueldeath = -1;
+        duelaffin = 0;
     }
 
     void reset()
@@ -25,26 +27,56 @@ struct duelservmode : servmode
         waitforhumans = true;
         shrink();
         duelqueue.shrink(0);
-        doreset();
+        restricted.shrink(0);
+        doreset(true);
         duelround = duelwins = 0;
         duelwinner = -1;
     }
 
-    void position(clientinfo *ci, int n)
+    void position()
     {
-        if(m_survivor(gamemode, mutators))
-            srvmsgft(ci->clientnum, CON_EVENT, "\fyyou are now \fs\fzgyqueued\fS for the \fs\fgnext match\fS");
-        else
+        loopv(clients)
         {
-            if(n) srvmsgft(ci->clientnum, CON_EVENT, "\fyyou are \fs\fzcy#%d\fS in the \fs\fgduel queue\fS", n+1);
-            else srvmsgft(ci->clientnum, CON_EVENT, "\fyyou are \fs\fzcrNEXT\fS in the \fs\fgduel queue\fS");
+            clientinfo *ci = clients[i];
+            if(m_survivor(gamemode, mutators)) ci->queuepos = -1;
+            else
+            {
+                int n = duelqueue.find(ci);
+                ci->queuepos = n > 0 && dueltime >= 0 ? n-1 : n;
+            }
+            sendf(-1, 1, "ri3", N_QUEUEPOS, ci->clientnum, ci->queuepos);
         }
     }
 
-    void queue(clientinfo *ci, bool pos = true, bool top = false, bool wait = true)
+    bool remqueue(clientinfo *ci, bool pos = true)
     {
-        if(ci->state.actortype >= A_ENEMY || ci->state.state == CS_SPECTATOR) return;
-        if(ci->state.actortype == A_PLAYER && waitforhumans) waitforhumans = false;
+        int n = duelqueue.find(ci);
+        if(n >= 0)
+        {
+            duelqueue.remove(n);
+            if(pos) position();
+            return true;
+        }
+        return false;
+    }
+
+    bool queue(clientinfo *ci, bool pos = true, bool top = false, bool wait = true)
+    {
+        if(ci->actortype >= A_ENEMY || ci->state == CS_SPECTATOR) return remqueue(ci, pos);
+        if(gamestate == G_S_OVERTIME && !restricted.empty() && restricted.find(ci) < 0) return remqueue(ci, pos);
+        if(DSGS(maxqueued) && duelqueue.find(ci) < 0 && playing.find(ci) < 0)
+        {
+            int count = 0;
+            loopv(duelqueue) if(duelqueue[i]->actortype == A_PLAYER) count++;
+            loopv(playing) if(playing[i]->actortype == A_PLAYER) count++;
+            if(count >= DSGS(maxqueued))
+            {
+                spectator(ci);
+                srvmsgft(ci->clientnum, CON_EVENT, "\fysorry, the \fs\fcqueue\fS is \fs\fzgcFULL\fS (max: \fs\fc%d\fS %s)", DSGS(maxqueued), DSGS(maxqueued) != 1 ? "players" : "player");
+                return remqueue(ci, pos);
+            }
+        }
+        if(ci->actortype == A_PLAYER && waitforhumans) waitforhumans = false;
         int n = duelqueue.find(ci);
         if(top)
         {
@@ -57,22 +89,18 @@ struct duelservmode : servmode
             n = duelqueue.length();
             duelqueue.add(ci);
         }
+        if(wait && ci->state != CS_WAITING) waiting(ci, DROP_RESET);
         if(allowed.find(ci) >= 0) allowed.removeobj(ci);
-        if(respawns.find(ci) >= 0) respawns.removeobj(ci);
-        if(wait && ci->state.state != CS_WAITING) waiting(ci, DROP_RESET);
-        if(pos) position(ci, n);
+        if(pos) position();
+        return true;
     }
 
     void entergame(clientinfo *ci)
     {
         queue(ci);
-        if(dueltime < 0 && dueldeath < 0 && m_affinity(gamemode)) switch(G(duelaffinity))
-        {
-            case 2: allowed.add(ci); // fall through because they need to be in respawns too
-            case 1: respawns.add(ci); break;
-            case 0: default: break;
-        }
+        if(dueltime < 0 && dueldeath < 0 && m_affinity(gamemode) && (DSGS(affinity) || numclients() <= 1)) allowed.add(ci);
     }
+
     void leavegame(clientinfo *ci, bool disconnecting = false)
     {
         if(duelwinner == ci->clientnum)
@@ -80,51 +108,26 @@ struct duelservmode : servmode
             duelwinner = -1;
             duelwins = 0;
         }
-        duelqueue.removeobj(ci);
         allowed.removeobj(ci);
         playing.removeobj(ci);
-        respawns.removeobj(ci);
+        remqueue(ci);
     }
 
     bool damage(clientinfo *m, clientinfo *v, int damage, int weap, int flags, int material, const ivec &hitpush, const ivec &hitvel, float dist)
     {
-        if(dueltime >= 0 && m->state.actortype < A_ENEMY) return false;
+        if(dueltime >= 0) return false;
         return true;
     }
 
     bool canspawn(clientinfo *ci, bool tryspawn = false)
     {
-        if(allowed.find(ci) >= 0 || ci->state.actortype >= A_ENEMY) return true;
+        if(allowed.find(ci) >= 0 || ci->actortype >= A_ENEMY) return true;
+        if(gamestate == G_S_OVERTIME && !restricted.empty() && restricted.find(ci) < 0) return false;
         else if(tryspawn)
         {
-            if(!m_affinity(gamemode)) queue(ci);
+            queue(ci);
             return true;
         }
-        if(m_affinity(gamemode) && respawns.find(ci) >= 0)
-        {
-            int delay = m_xdelay(gamemode, mutators, ci->team);
-            if(delay && ci->state.respawnwait(gamemillis, delay)) return false;
-            if(m_survivor(gamemode, mutators))
-            {
-                int alive = 0;
-                vector<clientinfo *> mates;
-                loopv(clients) if(clients[i]->state.actortype < A_ENEMY && clients[i]->team == ci->team)
-                { // includes ci
-                    mates.add(clients[i]);
-                    if(clients[i]->state.state == CS_ALIVE && (!G(duelbotcheck) || clients[i]->state.actortype != A_BOT)) alive++;
-                }
-                if(!alive)
-                {
-                    loopv(mates) if(allowed.find(mates[i]) < 0) allowed.add(mates[i]);
-                    duelcheck = gamemillis+1000;
-                    return true;
-                }
-                return false; // only respawn when all dead
-            }
-            if(allowed.find(ci) < 0) allowed.add(ci);
-            duelcheck = gamemillis+1000;
-            return true; // overrides it
-        }
         return false; // you spawn when we want you to buddy
     }
 
@@ -135,58 +138,59 @@ struct duelservmode : servmode
 
     void layout()
     {
-        loopvj(clients) if(clients[j]->state.actortype < A_ENEMY)
+        loopvj(clients)
         {
             vector<int> shots;
             loop(a, W_MAX) loop(b, 2)
             {
-                loopv(clients[j]->state.weapshots[a][b].projs)
-                    shots.add(clients[j]->state.weapshots[a][b].projs[i].id);
-                clients[j]->state.weapshots[a][b].projs.shrink(0);
+                loopv(clients[j]->weapshots[a][b].projs)
+                    shots.add(clients[j]->weapshots[a][b].projs[i].id);
+                clients[j]->weapshots[a][b].projs.shrink(0);
             }
             if(!shots.empty()) sendf(-1, 1, "ri2iv", N_DESTROY, clients[j]->clientnum, shots.length(), shots.length(), shots.getbuf());
 
         }
-        if(m_survivor(gamemode, mutators) || G(duelclear))
+        if(DSGS(clear))
             loopv(sents) if(enttype[sents[i].type].usetype == EU_ITEM) setspawn(i, hasitem(i), true, true);
     }
 
     void scoreaffinity(clientinfo *ci, bool win)
     {
         if(!m_affinity(gamemode) || dueltime >= 0 || duelround <= 0) return;
-        respawns.shrink(0);
         #define scoredteam(x,y) (win ? x != y : x == y)
-        loopv(clients) if(clients[i]->state.actortype < A_ENEMY) switch(clients[i]->state.state)
+        int queued = 0;
+        loopv(clients) if(clients[i]->actortype < A_ENEMY) switch(clients[i]->state)
         {
             case CS_ALIVE:
-                if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team)) queue(clients[i], false);
+                if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team)) if(queue(clients[i], false)) queued++;
                 break;
             case CS_DEAD:
                 if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team))
                 {
-                    queue(clients[i], false);
+                    if(queue(clients[i], false)) queued++;
                     break;
                 }
                 if(allowed.find(clients[i]) < 0) allowed.add(clients[i]);
                 waiting(clients[i], DROP_RESET);
-                duelcheck = gamemillis+1000;
                 break;
             case CS_WAITING:
                 if(playing.find(clients[i]) < 0 || scoredteam(clients[i]->team, ci->team))
                 {
-                    queue(clients[i], false);
+                    if(queue(clients[i], false)) queued++;
                     break;
                 }
                 if(allowed.find(clients[i]) < 0) allowed.add(clients[i]);
-                duelcheck = gamemillis+1000;
                 break;
             default: break;
         }
+        duelaffin = win ? ci->team : -ci->team;
+        duelcheck = gamemillis+DSGS(delay);
+        if(queued) position();
     }
 
     void clear()
     {
-        doreset();
+        doreset(false);
         bool reset = false;
         if(m_duel(gamemode, mutators) && G(duelcycle)&(m_team(gamemode, mutators) ? 2 : 1) && duelwinner >= 0 && duelwins > 0)
         {
@@ -195,7 +199,7 @@ struct duelservmode : servmode
             {
                 int numwins = G(duelcycles), numplrs = 0;
                 loopv(clients)
-                    if(clients[i]->state.actortype < A_ENEMY && clients[i]->state.state != CS_SPECTATOR && clients[i]->team == ci->team)
+                    if(clients[i]->actortype < A_ENEMY && clients[i]->state != CS_SPECTATOR && clients[i]->team == ci->team)
                         numplrs++;
                 if(numplrs > (m_team(gamemode, mutators) ? 1 : 2))
                 {
@@ -209,8 +213,21 @@ struct duelservmode : servmode
                 duelwins = 0;
             }
         }
-        loopv(clients) queue(clients[i], false, !reset && clients[i]->state.state == CS_ALIVE, reset || G(duelreset) || clients[i]->state.state != CS_ALIVE);
+        int queued = 0;
+        loopv(clients) if(queue(clients[i], false, !reset && clients[i]->state == CS_ALIVE, reset || DSGS(reset) || clients[i]->state != CS_ALIVE)) queued++;
         shrink();
+        if(queued) position();
+    }
+
+    void endffaround(vector<clientinfo *> alive)
+    {
+        loopv(clients) if(playing.find(clients[i]) >= 0)
+        {
+            ffaroundstats rs;
+            rs.round = duelround;
+            rs.winner = !alive.empty() && clients[i] == alive[0];
+            clients[i]->ffarounds.add(rs);
+        }
     }
 
     void update()
@@ -223,13 +240,39 @@ struct duelservmode : servmode
             {
                 shrink();
                 int wants = max(numteams(gamemode, mutators), 2);
-                loopv(clients) if(clients[i]->state.state != CS_SPECTATOR && clients[i]->state.state != CS_ALIVE) queue(clients[i], false);
+                loopv(clients) if(clients[i]->state != CS_SPECTATOR && clients[i]->state != CS_ALIVE) queue(clients[i], false);
+                if(gamestate == G_S_OVERTIME && DSGS(overtime) && m_ffa(gamemode, mutators) && restricted.empty())
+                {
+                    loopv(clients)
+                    {
+                        clientinfo *cs = clients[i];
+                        if(cs->state != CS_ALIVE && cs->state != CS_WAITING)
+                        {
+                            remqueue(cs, false);
+                            continue;
+                        }
+                        if(restricted.empty() || cs->points > restricted[0]->points)
+                        {
+                            restricted.shrink(0);
+                            restricted.add(cs);
+                        }
+                        else if(cs->points == restricted[0]->points) restricted.add(cs);
+                    }
+                    loopv(clients)
+                    {
+                        clientinfo *cs = clients[i];
+                        if(duelqueue.find(cs) >= 0 && restricted.find(cs) < 0)
+                        {
+                            remqueue(cs, false);
+                        }
+                    }
+                }
                 loopv(duelqueue)
                 {
                     if(m_duel(gamemode, mutators) && playing.length() >= wants) break;
-                    if(duelqueue[i]->state.state != CS_ALIVE)
+                    if(duelqueue[i]->state != CS_ALIVE)
                     {
-                        if(duelqueue[i]->state.state != CS_WAITING) waiting(duelqueue[i], DROP_RESET);
+                        if(duelqueue[i]->state != CS_WAITING) waiting(duelqueue[i], DROP_RESET);
                         if(m_duel(gamemode, mutators) && m_team(gamemode, mutators))
                         {
                             bool skip = false;
@@ -255,46 +298,50 @@ struct duelservmode : servmode
                             else if(i == wants-2) concatstring(names, " and ");
                             else concatstring(names, ", ");
                         }
-                        formatstring(fight)("\fyduel between %s, round \fs\fr#%d\fS", names, duelround);
+                        formatstring(fight, "duel between %s, round \fs\fc#%d\fS", names, duelround);
                     }
-                    else if(m_survivor(gamemode, mutators)) formatstring(fight)("\fysurvivor, round \fs\fr#%d\fS", duelround);
+                    else if(m_survivor(gamemode, mutators)) formatstring(fight, "survivor, round \fs\fc#%d\fS", duelround);
                     loopv(playing)
                     {
-                        if(playing[i]->state.state == CS_ALIVE)
+                        if(playing[i]->state == CS_ALIVE)
                         {
-                            playing[i]->state.lastregen = gamemillis;
-                            playing[i]->state.resetresidual();
-                            playing[i]->state.health = m_health(gamemode, mutators, playing[i]->state.model);
-                            sendf(-1, 1, "ri4", N_REGEN, playing[i]->clientnum, playing[i]->state.health, 0); // amt = 0 regens impulse
+                            playing[i]->lastregen = gamemillis;
+                            playing[i]->lastregenamt = 0; // amt = 0 regens impulse
+                            playing[i]->resetresidual();
+                            playing[i]->health = m_health(gamemode, mutators, playing[i]->actortype);
+                            sendf(-1, 1, "ri4", N_REGEN, playing[i]->clientnum, playing[i]->health, playing[i]->lastregenamt);
                         }
                         else if(allowed.find(playing[i]) < 0) allowed.add(playing[i]);
                         duelqueue.removeobj(playing[i]);
-                        if(m_affinity(gamemode)) respawns.add(playing[i]);
                     }
-                    loopv(duelqueue) position(duelqueue[i], i);
-                    ancmsgft(-1, S_V_FIGHT, CON_INFO, fight);
+                    if(gamestate == G_S_OVERTIME && !restricted.empty())
+                        ancmsgft(-1, S_V_FIGHT, CON_SELF, "\fy\fs\fzcgsudden death\fS %s", fight);
+                    else ancmsgft(-1, S_V_FIGHT, CON_SELF, "\fy%s", fight);
                     dueltime = dueldeath = -1;
                     duelcheck = gamemillis+5000;
                 }
                 else shrink();
+                position();
             }
         }
         else if(duelround > 0)
         {
             bool cleanup = false;
             vector<clientinfo *> alive;
-            loopv(clients) if(clients[i]->state.actortype < A_ENEMY && clients[i]->state.state == CS_ALIVE)
+            int queued = 0;
+            loopv(clients) if(clients[i]->actortype < A_ENEMY && clients[i]->state == CS_ALIVE)
             {
-                if(playing.find(clients[i]) < 0) queue(clients[i]);
+                if(playing.find(clients[i]) < 0) { if(queue(clients[i], false)) queued++; }
                 else alive.add(clients[i]);
             }
+            if(queued) position();
             if(!allowed.empty() && duelcheck >= 0 && gamemillis >= duelcheck) loopvrev(allowed)
             {
                 if(alive.find(allowed[i]) < 0) spectator(allowed[i]);
                 allowed.remove(i);
                 cleanup = true;
             }
-            if(allowed.empty() && (!m_affinity(gamemode) || respawns.empty()))
+            if(allowed.empty())
             {
                 if(m_survivor(gamemode, mutators) && m_team(gamemode, mutators) && !alive.empty())
                 {
@@ -302,27 +349,37 @@ struct duelservmode : servmode
                     loopv(alive) if(i && alive[i]->team != alive[i-1]->team) { found = true; break; }
                     if(!found)
                     {
-                        if(dueldeath < 0) dueldeath = gamemillis+G(dueldelay);
+                        if(dueldeath < 0) dueldeath = gamemillis+DSGS(delay);
                         else if(gamemillis >= dueldeath)
                         {
                             if(!cleanup)
                             {
-                                defformatstring(end)("\fyteam %s are the winners", colourteam(alive[0]->team));
+                                defformatstring(end, "\fyteam %s are the winners", colourteam(alive[0]->team));
                                 bool teampoints = true;
                                 loopv(clients) if(playing.find(clients[i]) >= 0)
                                 {
                                     if(clients[i]->team == alive[0]->team)
                                     {
-                                        ancmsgft(clients[i]->clientnum, S_V_YOUWIN, CON_INFO, end);
-                                        if(!m_affinity(gamemode) && alive.find(clients[i]) >= 0)
+                                        ancmsgft(clients[i]->clientnum, S_V_YOUWIN, CON_SELF, "%s", end);
+                                        if(alive.find(clients[i]) >= 0)
                                         {
-                                            givepoints(clients[i], 1, teampoints);
-                                            teampoints = false;
+                                            if(!m_affinity(gamemode))
+                                            {
+                                                givepoints(clients[i], 1, true, teampoints);
+                                                teampoints = false;
+                                            }
+                                            else if(!duelaffin && teampoints)
+                                            {
+                                                score &ts = teamscore(clients[i]->team);
+                                                ts.total++;
+                                                sendf(-1, 1, "ri3", N_SCORE, ts.team, ts.total);
+                                                teampoints = false;
+                                            }
                                         }
                                     }
-                                    else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_INFO, end);
+                                    else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_SELF, "%s", end);
                                 }
-                                else ancmsgft(clients[i]->clientnum, S_V_BOMBSCORE, CON_INFO, end);
+                                else ancmsgft(clients[i]->clientnum, S_V_BOMBSCORE, CON_SELF, "%s", end);
                             }
                             clear();
                         }
@@ -332,10 +389,17 @@ struct duelservmode : servmode
                 {
                     case 0:
                     {
-                        if(m_affinity(gamemode) && !playing.empty()) break; // this should not happen
+                        if(m_affinity(gamemode) && duelaffin) // reverse it!
+                        {
+                            score &ts = teamscore(abs(duelaffin));
+                            if(duelaffin > 0) ts.total--;
+                            else ts.total++;
+                            sendf(-1, 1, "ri3", N_SCORE, ts.team, ts.total);
+                        }
                         if(!cleanup)
                         {
-                            ancmsgft(-1, S_V_DRAW, CON_INFO, "\fyeveryone died, \fzoyepic fail");
+                            endffaround(alive);
+                            ancmsgft(-1, S_V_DRAW, CON_SELF, "\fyeveryone died, \fzoyepic fail");
                             duelwinner = -1;
                             duelwins = 0;
                         }
@@ -344,34 +408,32 @@ struct duelservmode : servmode
                     }
                     case 1:
                     {
-                        if(!m_affinity(gamemode))
+                        if(dueldeath < 0)
                         {
-                            if(dueldeath < 0)
-                            {
-                                dueldeath = gamemillis+G(dueldelay);
-                                break;
-                            }
-                            else if(gamemillis < dueldeath) break;
+                            dueldeath = gamemillis+DSGS(delay);
+                            break;
                         }
+                        else if(gamemillis < dueldeath) break;
                         if(!cleanup)
                         {
-                            string end, hp; hp[0] = 0;
-                            if(!m_insta(gamemode, mutators) && !m_affinity(gamemode))
+                            endffaround(alive);
+                            string end = "", hp = "";
+                            if(!m_insta(gamemode, mutators))
                             {
-                                if(alive[0]->state.health >= m_health(gamemode, mutators, alive[0]->state.model))
-                                    formatstring(hp)(" with a \fs\fcflawless victory\fS");
-                                else formatstring(hp)(" with \fs\fc%d\fS health left", alive[0]->state.health);
+                                if(alive[0]->health >= m_health(gamemode, mutators, alive[0]->actortype))
+                                    formatstring(hp, " with a \fs\fcflawless victory\fS");
+                                else formatstring(hp, " with \fs\fc%d\fS health left", alive[0]->health);
                             }
                             if(duelwinner != alive[0]->clientnum)
                             {
                                 duelwinner = alive[0]->clientnum;
                                 duelwins = 1;
-                                formatstring(end)("\fy%s was the winner%s", colourname(alive[0]), hp);
+                                formatstring(end, "\fy%s was the winner%s", colourname(alive[0]), hp);
                             }
                             else
                             {
                                 duelwins++;
-                                formatstring(end)("\fy%s was the winner%s (\fs\fc%d\fS in a row)", colourname(alive[0]), hp, duelwins);
+                                formatstring(end, "\fy%s was the winner%s (\fs\fc%d\fS in a row)", colourname(alive[0]), hp, duelwins);
                             }
                             loopv(clients)
                             {
@@ -379,12 +441,18 @@ struct duelservmode : servmode
                                 {
                                     if(clients[i] == alive[0])
                                     {
-                                        ancmsgft(clients[i]->clientnum, S_V_YOUWIN, CON_INFO, end);
-                                        if(!m_affinity(gamemode)) givepoints(clients[i], 1);
+                                        ancmsgft(clients[i]->clientnum, S_V_YOUWIN, CON_SELF, "%s", end);
+                                        if(!m_affinity(gamemode)) givepoints(clients[i], 1, true, true);
+                                        else if(!duelaffin)
+                                        {
+                                            score &ts = teamscore(clients[i]->team);
+                                            ts.total++;
+                                            sendf(-1, 1, "ri3", N_SCORE, ts.team, ts.total);
+                                        }
                                     }
-                                    else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_INFO, end);
+                                    else ancmsgft(clients[i]->clientnum, S_V_YOULOSE, CON_SELF, "%s", end);
                                 }
-                                else ancmsgft(clients[i]->clientnum, S_V_BOMBSCORE, CON_INFO, end);
+                                else ancmsgft(clients[i]->clientnum, S_V_BOMBSCORE, CON_SELF, "%s", end);
                             }
                         }
                         clear();
@@ -410,7 +478,7 @@ struct duelservmode : servmode
 
     void balance(int oldbalance)
     {
-        doreset();
+        doreset(true);
     }
 } duelmutator;
 #endif
