@@ -1,7 +1,7 @@
 enum
 {
     G_DEMO = 0, G_EDITMODE, G_DEATHMATCH, G_CAPTURE, G_DEFEND, G_BOMBER, G_RACE, G_MAX,
-    G_START = G_EDITMODE, G_PLAY = G_DEATHMATCH, G_FIGHT = G_DEATHMATCH,
+    G_START = G_EDITMODE, G_PLAY = G_DEATHMATCH,
     G_RAND = G_BOMBER-G_DEATHMATCH+1, G_COUNT = G_MAX-G_PLAY,
     G_NEVER = (1<<G_DEMO)|(1<<G_EDITMODE),
     G_LIMIT = (1<<G_DEATHMATCH)|(1<<G_CAPTURE)|(1<<G_DEFEND)|(1<<G_BOMBER),
@@ -23,9 +23,19 @@ enum
 };
 enum { G_F_GSP = 0, G_F_NUM };
 
-enum { G_S_WAITING = 0, G_S_VOTING, G_S_INTERMISSION, G_S_PLAYING, G_S_OVERTIME, G_S_MAX };
-#define gs_playing(a) (a == G_S_PLAYING || a == G_S_OVERTIME)
-#define gs_intermission(a) (a == G_S_INTERMISSION || a == G_S_VOTING)
+enum { G_S_WAITING = 0, G_S_GETMAP, G_S_SENDMAP, G_S_READYING, G_S_GAMEINFO, G_S_PLAYING, G_S_OVERTIME, G_S_INTERMISSION, G_S_VOTING, G_S_MAX };
+#ifdef GAMESERVER
+const char *gamestates[3][G_S_MAX] = {
+    { "waiting", "getmap", "sendmap", "readying", "gameinfo", "playing", "overtime", "intermission", "voting" },
+    { "waiting to start", "server getting map", "server sending map", "waiting for ready players", "waiting for game information", "playing", "overtime", "intermission", "voting in progress" },
+    { "Waiting to start", "Server getting map", "Server sending map", "Waiting for ready players", "Waiting for game information", "Playing", "Overtime", "Intermission", "Voting in progress" }
+};
+#else
+extern const char *gamestates[3][G_S_MAX];
+#endif
+#define gs_waiting(a) (a >= G_S_WAITING && a <= G_S_GAMEINFO)
+#define gs_playing(a) (a >= G_S_PLAYING && a <= G_S_OVERTIME)
+#define gs_intermission(a) (a >= G_S_INTERMISSION && a <= G_S_VOTING)
 
 struct gametypes
 {
@@ -56,11 +66,13 @@ gametypes gametype[] = {
     {
         G_DEATHMATCH, 0, 0,
         {
-            (1<<G_M_MULTI)|(1<<G_M_FFA)|(1<<G_M_COOP)|(1<<G_M_INSTA)|(1<<G_M_DUEL)|(1<<G_M_SURVIVOR)|(1<<G_M_CLASSIC)|(1<<G_M_MEDIEVAL)|(1<<G_M_KABOOM)|(1<<G_M_ONSLAUGHT)|(1<<G_M_FREESTYLE)|(1<<G_M_VAMPIRE)|(1<<G_M_RESIZE)|(1<<G_M_HARD)|(1<<G_M_BASIC),
-            0, 0, 0
+            (1<<G_M_MULTI)|(1<<G_M_FFA)|(1<<G_M_COOP)|(1<<G_M_INSTA)|(1<<G_M_DUEL)|(1<<G_M_SURVIVOR)|(1<<G_M_CLASSIC)|(1<<G_M_MEDIEVAL)|(1<<G_M_KABOOM)|(1<<G_M_ONSLAUGHT)|(1<<G_M_FREESTYLE)|(1<<G_M_VAMPIRE)|(1<<G_M_RESIZE)|(1<<G_M_HARD)|(1<<G_M_BASIC)|(1<<G_M_GSP1)|(1<<G_M_GSP2),
+            (1<<G_M_MULTI)|(1<<G_M_FFA)|(1<<G_M_COOP)|(1<<G_M_INSTA)|(1<<G_M_DUEL)|(1<<G_M_SURVIVOR)|(1<<G_M_CLASSIC)|(1<<G_M_MEDIEVAL)|(1<<G_M_KABOOM)|(1<<G_M_ONSLAUGHT)|(1<<G_M_FREESTYLE)|(1<<G_M_VAMPIRE)|(1<<G_M_RESIZE)|(1<<G_M_HARD)|(1<<G_M_BASIC)|(1<<G_M_GSP1),
+            (1<<G_M_MULTI)|(1<<G_M_FFA)|(1<<G_M_COOP)|(1<<G_M_INSTA)|(1<<G_M_CLASSIC)|(1<<G_M_MEDIEVAL)|(1<<G_M_KABOOM)|(1<<G_M_ONSLAUGHT)|(1<<G_M_FREESTYLE)|(1<<G_M_VAMPIRE)|(1<<G_M_RESIZE)|(1<<G_M_HARD)|(1<<G_M_BASIC)|(1<<G_M_GSP2),
+            0
         },
-        "deathmatch", "dm", { "", "", "" },
-        "shoot to kill and increase score by fragging", { "", "", "" },
+        "deathmatch", "dm", { "gladiator", "oldschool", "" },
+        "shoot to kill and increase score by fragging", { "fight in a confined area with increased pushback from damage", "secure the most number of frags instead of points", "" },
     },
     {
         G_CAPTURE, 0, 0,
@@ -151,8 +163,7 @@ mutstypes mutstype[] = {
     {
         G_M_CLASSIC,    (1<<G_M_CLASSIC),
         (1<<G_M_MULTI)|(1<<G_M_FFA)|(1<<G_M_COOP)|(1<<G_M_DUEL)|(1<<G_M_SURVIVOR)|(1<<G_M_CLASSIC)|(1<<G_M_ONSLAUGHT)|(1<<G_M_FREESTYLE)|(1<<G_M_VAMPIRE)|(1<<G_M_RESIZE)|(1<<G_M_HARD)|(1<<G_M_BASIC)|(1<<G_M_GSP1)|(1<<G_M_GSP2)|(1<<G_M_GSP3),
-        "classic",
-        "weapons must be collected from spawns in the arena"
+        "classic", "weapons must be collected from spawns in the arena"
     },
     {
         G_M_ONSLAUGHT, (1<<G_M_ONSLAUGHT),
@@ -177,7 +188,7 @@ mutstypes mutstype[] = {
     {
         G_M_HARD, (1<<G_M_HARD),
         (1<<G_M_MULTI)|(1<<G_M_FFA)|(1<<G_M_COOP)|(1<<G_M_INSTA)|(1<<G_M_MEDIEVAL)|(1<<G_M_KABOOM)|(1<<G_M_DUEL)|(1<<G_M_SURVIVOR)|(1<<G_M_CLASSIC)|(1<<G_M_ONSLAUGHT)|(1<<G_M_FREESTYLE)|(1<<G_M_VAMPIRE)|(1<<G_M_RESIZE)|(1<<G_M_HARD)|(1<<G_M_BASIC)|(1<<G_M_GSP1)|(1<<G_M_GSP2)|(1<<G_M_GSP3),
-        "hard", "no regeneration, no radar, no impulse reset"
+        "hard", "no health regeneration, no radar"
     },
     {
         G_M_BASIC, (1<<G_M_BASIC),
@@ -219,7 +230,6 @@ extern mutstypes mutstype[];
 
 #define m_play(a)           (a >= G_PLAY)
 #define m_affinity(a)       (m_capture(a) || m_defend(a) || m_bomber(a))
-#define m_fight(a)          (a >= G_FIGHT)
 
 #define m_multi(a,b)        ((b&(1<<G_M_MULTI)) || (gametype[a].implied&(1<<G_M_MULTI)))
 #define m_ffa(a,b)          ((b&(1<<G_M_FFA)) || (gametype[a].implied&(1<<G_M_FFA)))
@@ -235,43 +245,99 @@ extern mutstypes mutstype[];
 #define m_vampire(a,b)      ((b&(1<<G_M_VAMPIRE)) || (gametype[a].implied&(1<<G_M_VAMPIRE)))
 #define m_resize(a,b)       ((b&(1<<G_M_RESIZE)) || (gametype[a].implied&(1<<G_M_RESIZE)))
 #define m_hard(a,b)         ((b&(1<<G_M_HARD)) || (gametype[a].implied&(1<<G_M_HARD)))
-#define m_basic(a,b)         ((b&(1<<G_M_BASIC)) || (gametype[a].implied&(1<<G_M_BASIC)))
+#define m_basic(a,b)        ((b&(1<<G_M_BASIC)) || (gametype[a].implied&(1<<G_M_BASIC)))
 
 #define m_gsp1(a,b)         ((b&(1<<G_M_GSP1)) || (gametype[a].implied&(1<<G_M_GSP1)))
 #define m_gsp2(a,b)         ((b&(1<<G_M_GSP2)) || (gametype[a].implied&(1<<G_M_GSP2)))
 #define m_gsp3(a,b)         ((b&(1<<G_M_GSP3)) || (gametype[a].implied&(1<<G_M_GSP3)))
 #define m_gsp(a,b)          (m_gsp1(a,b) || m_gsp2(a,b) || m_gsp3(a,b))
 
+#define m_dm_gladiator(a,b) (m_dm(a) && m_gsp1(a, b))
+#define m_dm_oldschool(a,b) (m_dm(a) && m_gsp2(a, b))
+
+#define m_ctf_quick(a,b)    (m_capture(a) && m_gsp1(a, b))
+#define m_ctf_defend(a,b)   (m_capture(a) && m_gsp2(a, b))
+#define m_ctf_protect(a,b)  (m_capture(a) && m_gsp3(a, b))
+
+#define m_dac_quick(a,b)    (m_defend(a) && m_gsp1(a, b))
+#define m_dac_king(a,b)     (m_defend(a) && m_gsp2(a, b))
+
+#define m_bb_hold(a,b)      (m_bomber(a) && m_gsp1(a, b))
+#define m_bb_basket(a,b)    (m_bomber(a) && m_gsp2(a, b))
+#define m_bb_attack(a,b)    (m_bomber(a) && m_gsp3(a, b))
+
+#define m_ra_timed(a,b)     (m_race(a) && m_gsp1(a, b))
+#define m_ra_endurance(a,b) (m_race(a) && m_gsp2(a, b))
+#define m_ra_gauntlet(a,b)  (m_race(a) && m_gsp3(a, b))
+
 #define m_team(a,b)         (m_multi(a, b) || !m_ffa(a, b))
-#define m_sweaps(a,b)       (m_race(a) || m_insta(a, b) || m_medieval(a, b) || m_kaboom(a, b))
+#define m_sweaps(a,b)       (m_dm_gladiator(a, b) || (m_race(a) && !m_ra_gauntlet(a, b)) || m_insta(a, b) || m_medieval(a, b) || m_kaboom(a, b))
 #define m_loadout(a,b)      (!m_classic(a, b) && !m_sweaps(a, b))
 #define m_duke(a,b)         (m_duel(a, b) || m_survivor(a, b))
-#define m_regen(a,b)        (!m_hard(a,b) && (G(duelregen) || !m_duke(a, b)) && !m_insta(a, b))
-#define m_ghost(a,b)        (m_race(a) && !m_gsp3(a, b))
-#define m_bots(a)           (m_fight(a) && !m_race(a))
+#define m_regen(a,b)        (!m_hard(a,b) && (!m_duke(a, b) || DSG(a, b, regen)) && !m_insta(a, b))
+#define m_ghost(a,b)        (m_race(a) && !m_ra_gauntlet(a, b))
+#define m_bots(a)           (m_play(a) && !m_race(a))
 #define m_botbal(a,b)       (m_duel(a, b) ? G(botbalanceduel) : (m_survivor(a, b) ? G(botbalancesurvivor) : G(botbalance)))
-#define m_nopoints(a,b)     (m_duke(a, b) || (m_bomber(a) && m_gsp1(a, b)) || m_race(a))
-#define m_laptime(a,b)      (m_race(a) && m_gsp1(a, b))
-#define m_impulsemeter(a,b) ((m_race(a) && m_gsp2(a, b)) || !m_freestyle(a, b))
+#define m_laptime(a,b)      (m_ra_timed(a, b))
+#define m_impulsemeter(a,b) (m_ra_endurance(a, b) || !m_freestyle(a, b))
+#define m_nopoints(a,b)     (m_duke(a, b) || m_bb_hold(a, b) || m_race(a))
+#define m_points(a,b)       (!m_nopoints(a, b))
+#define m_usetotals(a,b)    (!m_race(a))
 
-#define m_weapon(a,b)       (m_medieval(a, b) ? W_SWORD : (m_kaboom(a, b) ? W_GRENADE : (m_insta(a, b) ? G(instaweapon) : (m_race(a) && !m_gsp3(a, b) ? G(raceweapon) : G(spawnweapon)))))
-#define m_xdelay(a,b,c)     (m_play(a) ? (m_race(a) ? (!m_gsp3(a, b) || c == T_ALPHA ? G(racedelay) : G(racedelayex)) : (m_bomber(a) ? G(bomberdelay) : (m_insta(a, b) ? G(instadelay) : G(spawndelay)))) : 0)
-#define m_delay(a,b,c)      (m_duke(a,b) ? 0 : m_xdelay(a, b, c))
-#define m_protect(a,b)      (m_duke(a,b) ? G(duelprotect) : (m_insta(a, b) ? G(instaprotect) : G(spawnprotect)))
-#define m_health(a,b,c)     (m_insta(a,b) ? 1 : PLAYER(c, health))
+#define m_weapon(at,a,b)    (m_medieval(a, b) ? AA(at, weaponmedieval) : (m_kaboom(a, b) ? AA(at, weaponkaboom) : (m_insta(a, b) ? AA(at, weaponinsta) : (m_race(a) && !m_ra_gauntlet(a, b) ? AA(at, weaponrace) : (m_dm_gladiator(a, b) ? AA(at, weapongladiator) : AA(at, weaponspawn))))))
+#define m_delay(at,a,b,c)   ((m_play(a) || at >= A_ENEMY) && !m_duke(a,b) ? int((m_race(a) ? (!m_ra_gauntlet(a, b) || c == T_ALPHA ? AA(at, spawndelayrace) : AA(at, spawndelaygauntlet)) : (m_bomber(a) ? AA(at, spawndelaybomber) : (m_defend(a) ? AA(at, spawndelaydefend) : (m_capture(a) ? AA(at, spawndelaycapture) : AA(at, spawndelay)))))*(m_insta(a, b) ? AA(at, spawndelayinstascale) : 1.f)) : 0)
+#define m_protect(a,b)      (m_duke(a,b) ? DSG(a, b, protect) : (m_insta(a, b) ? G(instaprotect) : G(spawnprotect)))
+#define m_health(a,b,c)     (m_insta(a,b) ? 1 : AA(c, health))
 #define m_maxhealth(a,b,c)  (int(m_health(a, b, c)*(m_vampire(a,b) ? G(maxhealthvampire) : G(maxhealth))))
-#define m_swapteam(a,b)     (m_team(a, b) && (!m_race(a) || m_gsp3(a, b)) && m_fight(a) && (G(teambalanceduel) || !m_duel(a, b)) && !m_coop(gamemode, mutators) && G(teambalance) >= 3 && G(teambalanceswap))
-#define m_balteam(a,b,c)    (m_team(a, b) && (!m_race(a) || m_gsp3(a, b)) && m_fight(a) && (G(teambalanceduel) || !m_duel(a, b)) && !m_coop(gamemode, mutators) && G(teambalance) >= c)
-#define m_forcebal(a,b)     ((m_bomber(a) && m_gsp3(a, b)) || (m_race(a) && m_gsp3(a, b)))
-#define m_balance(a,b,c)    (m_team(a, b) && (!m_race(a) || m_gsp3(a, b)) && m_fight(a) && (m_forcebal(a, b) || ((G(balanceduke) || !m_duke(a, b)) && ((G(balancemaps) >= 0 ? G(balancemaps) : G(mapbalance)) >= (m_affinity(a) ? 1 : (c ? 2 : 3))))))
+#define m_swapteam(a,b)     (m_team(a, b) && (!m_race(a) || m_ra_gauntlet(a, b)) && m_play(a) && (G(teambalanceduel) || !m_duel(a, b)) && !m_coop(gamemode, mutators) && G(teambalance) >= 3 && G(teambalanceswap))
+#define m_balteam(a,b,c)    (m_team(a, b) && (!m_race(a) || m_ra_gauntlet(a, b)) && m_play(a) && (G(teambalanceduel) || !m_duel(a, b)) && !m_coop(gamemode, mutators) && G(teambalance) >= c)
+#define m_forcebal(a,b)     (m_bb_attack(a, b) || m_ra_gauntlet(a, b))
+#define m_balance(a,b,c)    (m_team(a, b) && (!m_race(a) || m_ra_gauntlet(a, b)) && m_play(a) && (m_forcebal(a, b) || ((G(balanceduke) || !m_duke(a, b)) && ((G(balancemaps) >= 0 ? G(balancemaps) : G(mapbalance)) >= (m_affinity(a) ? 1 : (c ? 2 : 3))))))
 #define m_balreset(a,b)     (G(balancereset) && (G(balancereset) == 2 || m_capture(a) || m_bomber(a) || m_race(a) || m_duke(a, b)))
 
-#define w_carry(w1,w2)      (isweap(w1) && w1 != W_MELEE && (!isweap(w2) || (w1 != w2 && (w2 != W_GRENADE || w1 != W_MINE))) && (w1 == W_ROCKET || (w1 >= W_OFFSET && w1 < W_ITEM)))
-#define w_reload(w1,w2)     (isweap(w1) && (w1 == W_MELEE || (w1 >= W_OFFSET && w1 < W_ITEM) || (isweap(w2) && (w1 == w2 || (w2 == W_GRENADE && w1 == W_MINE)))))
-#define w_item(w1,w2)       (isweap(w1) && (w1 >= W_OFFSET && w1 < W_MAX && (!isweap(w2) || (w1 != w2 && (w2 != W_GRENADE || w1 != W_MINE)))))
-#define w_attr(a,b,t,w1,w2) (t != WEAPON || m_edit(a) ? w1 : (w1 != w2 ? (!m_classic(a, b) ? (w1 >= W_ITEM ? w1 : -1) : (w1 >= W_OFFSET && w1 < W_MAX ? w1 : -1)) : (w1 != W_GRENADE ? W_GRENADE : W_MINE)))
+#define w_carry(w1,w2)      (isweap(w1) && w1 != W_CLAW && w1 < W_ALL && (!isweap(w2) || (w1 != w2 && (w2 != W_GRENADE || w1 != W_MINE))) && (w1 == W_ROCKET || (w1 >= W_OFFSET && w1 < W_ITEM)))
+#define w_reload(w1,w2)     (isweap(w1) && (w1 >= W_ALL || (w1 >= W_OFFSET && w1 < W_ITEM) || (isweap(w2) && (w1 == w2 || (w2 == W_GRENADE && w1 == W_MINE)))))
+#define w_item(w1,w2)       (isweap(w1) && (w1 >= W_OFFSET && w1 < W_ALL && (!isweap(w2) || (w1 != w2 && (w2 != W_GRENADE || w1 != W_MINE)))))
+#define w_attr(a,b,t,w1,w2) (t != WEAPON || m_edit(a) ? w1 : (w1 != w2 ? (!m_classic(a, b) ? (w1 >= W_ITEM ? w1 : -1) : (w1 >= W_OFFSET && w1 < W_ALL ? w1 : -1)) : (w1 != W_GRENADE ? W_GRENADE : W_MINE)))
 #define w_spawn(weap)       int(ceilf(G(itemspawntime)*W(weap, frequency)))
 
+#define m_mmvar(a,b,c)      (m_dm(a) ? \
+                                (m_duel(a, b) ? G(c##duel) : \
+                                    (m_survivor(a, b) ? G(c##survivor) : \
+                                        (m_dm_gladiator(a, b) ? G(c##gladiator) : G(c)) \
+                                    ) \
+                                ) : \
+                                (m_capture(a) ? G(c##capture) : \
+                                    (m_defend(a) ? \
+                                        (m_dac_king(a, b) ? G(c##defendking) : G(c##defend)) : \
+                                        (m_bomber(a) ? (m_bb_hold(a, b) ? G(c##bomberhold) : G(c##bomber)) : \
+                                            (m_race(a) ? \
+                                                (m_ra_timed(a, b) ? G(c##racetimed) : \
+                                                    (m_ra_gauntlet(a, b) ? G(c##racegauntlet) : G(c##race)) \
+                                                ) : 0 \
+                                            ) \
+                                        ) \
+                                    ) \
+                                ) \
+                            )
+
+#define MMVAR(f,a,b,c,w01,w02,w03,w04,w05,w06,w07,w08,w09,w10,w11,w12) \
+    GVAR(f, a, b, w01, c); \
+    GVAR(f, a##duel, b, w02, c); \
+    GVAR(f, a##survivor, b, w03, c); \
+    GVAR(f, a##gladiator, b, w04, c); \
+    GVAR(f, a##capture, b, w05, c); \
+    GVAR(f, a##defend, b, w06, c); \
+    GVAR(f, a##defendking, b, w07, c); \
+    GVAR(f, a##bomber, b, w08, c); \
+    GVAR(f, a##bomberhold, b, w09, c); \
+    GVAR(f, a##race, b, w10, c); \
+    GVAR(f, a##racetimed, b, w11, c); \
+    GVAR(f, a##racegauntlet, b, w12, c);
+
+#define DSG(a,b,x)          (m_duel(a, b) ? G(duel##x) : G(survivor##x))
+#define GL(a,b,x)           (m_dm_gladiator(a, b) ? gladiator##x : x)
+
 #define mapshrink(a,b,c,d) if((a) && (b) && (c) && *(c)) \
 { \
     char *p = shrinklist(b, c, 1, d); \
@@ -283,34 +349,38 @@ extern mutstypes mutstype[];
 }
 #define mapcull(a,b,c,d,e,f) \
 { \
-    mapshrink(m_multi(b, c) && (m_capture(b) || (m_bomber(b) && !m_gsp1(b, c))), a, G(multimaps), false) \
+    mapshrink(m_multi(b, c) && (m_capture(b) || (m_bomber(b) && !m_bb_hold(b, c))), a, G(multimaps), false) \
     mapshrink(m_duel(b, c), a, G(duelmaps), false) \
-    if(d > 0 && e >= 2 && m_fight(b) && !m_duel(b, c)) \
+    if((d) > 0 && (e) >= 2 && m_play(b) && !m_duel(b, c)) \
     { \
-        mapshrink(G(smallmapmax) && d <= G(smallmapmax), a, G(smallmaps), false) \
-        else mapshrink(G(mediummapmax) && d <= G(mediummapmax), a, G(mediummaps), false) \
-        else mapshrink(G(mediummapmax) && d > G(mediummapmax), a, G(largemaps), false) \
+        mapshrink(G(smallmapmax) && (d) <= G(smallmapmax), a, G(smallmaps), false) \
+        else mapshrink(G(mediummapmax) && (d) <= G(mediummapmax), a, G(mediummaps), false) \
+        else mapshrink(G(mediummapmax) && (d) > G(mediummapmax), a, G(largemaps), false) \
     } \
-    mapshrink(!f, a, G(previousmaps), true) \
+    mapshrink(!(f), a, G(previousmaps), true) \
 }
 #define maplist(a,b,c,d,e,f) \
 { \
     if(m_capture(b)) a = newstring(G(capturemaps)); \
-    else if(m_defend(b)) a = newstring(m_gsp2(b, c) ? G(kingmaps) : G(defendmaps)); \
-    else if(m_bomber(b)) a = newstring(m_gsp1(b, c) ? G(holdmaps) : G(bombermaps)); \
+    else if(m_defend(b)) a = newstring(m_dac_king(b, c) ? G(kingmaps) : G(defendmaps)); \
+    else if(m_bomber(b)) a = newstring(m_bb_hold(b, c) ? G(holdmaps) : G(bombermaps)); \
     else if(m_race(b)) a = newstring(G(racemaps)); \
-    else if(m_fight(b)) a = newstring(G(mainmaps)); \
+    else if(m_dm(b)) a = newstring(m_dm_gladiator(b, c) ? G(gladiatormaps) : G(mainmaps)); \
     else a = newstring(G(allowmaps)); \
     if(e) mapcull(a, b, c, d, e, f) \
-    else mapshrink(!f, a, G(previousmaps), true) \
+    else mapshrink(!(f), a, G(previousmaps), true) \
 }
 #ifdef GAMESERVER
-SVAR(0, gamestatename, "waiting voting intermission playing overtime");
+SVAR(0, gamestatename, "waiting getmap sendmap readying gameinfo playing overtime intermission voting");
 VAR(0, gamestatewaiting, 1, G_S_WAITING, -1);
-VAR(0, gamestatevoting, 1, G_S_VOTING, -1);
-VAR(0, gamestateintermission, 1, G_S_INTERMISSION, -1);
+VAR(0, gamestategetmap, 1, G_S_GETMAP, -1);
+VAR(0, gamestatesendmap, 1, G_S_SENDMAP, -1);
+VAR(0, gamestatereadying, 1, G_S_READYING, -1);
+VAR(0, gamestategameinfo, 1, G_S_GAMEINFO, -1);
 VAR(0, gamestateplaying, 1, G_S_PLAYING, -1);
 VAR(0, gamestateovertime, 1, G_S_OVERTIME, -1);
+VAR(0, gamestateintermission, 1, G_S_INTERMISSION, -1);
+VAR(0, gamestatevoting, 1, G_S_VOTING, -1);
 VAR(0, gamestatenum, 1, G_S_MAX, -1);
 SVAR(0, modename, "demo editing deathmatch capture-the-flag defend-and-control bomber-ball race");
 SVAR(0, modeidxname, "demo editing deathmatch capture defend bomber race");
@@ -323,7 +393,6 @@ VAR(0, modeidxbomber, 1, G_BOMBER, -1);
 VAR(0, modeidxrace, 1, G_RACE, -1);
 VAR(0, modeidxstart, 1, G_START, -1);
 VAR(0, modeidxplay, 1, G_PLAY, -1);
-VAR(0, modeidxfight, 1, G_FIGHT, -1);
 VAR(0, modeidxrand, 1, G_RAND, -1);
 VAR(0, modeidxnever, 1, G_NEVER, -1);
 VAR(0, modeidxlimit, 1, G_LIMIT, -1);
