@@ -12,25 +12,25 @@ namespace ai
     bool clipped(const vec &o)
     {
         int material = lookupmaterial(o), clipmat = material&MATF_CLIP;
-        return clipmat == MAT_CLIP || clipmat == MAT_AICLIP || material&MAT_DEATH || (material&MATF_VOLUME) == MAT_LAVA;
+        return clipmat == MAT_CLIP || clipmat == MAT_AICLIP || material&MAT_DEATH || material&MAT_HURT || (material&MATF_VOLUME) == MAT_LAVA;
     }
 
-    int getweight(const vec &o)
+    int getpull(const vec &o)
     {
         vec pos = o; pos.z += JUMPMIN;
         if(!insideworld(vec(pos.x, pos.y, min(pos.z, getworldsize() - 1e-3f)))) return -2;
         float dist = raycube(pos, vec(0, 0, -1), 0, RAY_CLIPMAT);
-        int posmat = lookupmaterial(pos), weight = 1;
-        if(isliquid(posmat&MATF_VOLUME)) weight *= 5;
+        int posmat = lookupmaterial(pos), pull = 1;
+        if(isliquid(posmat&MATF_VOLUME)) pull *= 5;
         if(dist >= 0)
         {
-            weight = int(dist/JUMPMIN);
+            pull = int(dist/JUMPMIN);
             pos.z -= clamp(dist-8.0f, 0.0f, pos.z);
             int trgmat = lookupmaterial(pos);
-            if(trgmat&MAT_DEATH || (trgmat&MATF_VOLUME) == MAT_LAVA) weight *= 10;
-            else if(isliquid(trgmat&MATF_VOLUME)) weight *= 2;
+            if(trgmat&MAT_DEATH || trgmat&MAT_HURT || (trgmat&MATF_VOLUME) == MAT_LAVA) pull *= 100;
+            else if(isliquid(trgmat&MATF_VOLUME)) pull *= 2;
         }
-        return weight;
+        return pull;
     }
 
     enum
@@ -176,7 +176,7 @@ namespace ai
     }
 
     void clearwpcache(bool full = true)
-	{
+    {
         loopi(NUMWPCACHES) if(full || invalidatedwpcaches&(1<<i)) { wpcaches[i].clear(); clearedwpcaches |= 1<<i; }
         if(full || invalidatedwpcaches == (1<<NUMWPCACHES)-1)
         {
@@ -184,7 +184,7 @@ namespace ai
             lastwpcache = 0;
         }
         invalidatedwpcaches = 0;
-	}
+    }
     ICOMMAND(0, clearwpcache, "", (), clearwpcache());
 
     void buildwpcache()
@@ -194,8 +194,8 @@ namespace ai
         clearedwpcaches = 0;
         lastwpcache = waypoints.length();
 
-		wpavoid.clear();
-		loopv(waypoints) if(waypoints[i].weight < 0) wpavoid.avoidnear(NULL, waypoints[i].o.z + WAYPOINTRADIUS, waypoints[i].o, WAYPOINTRADIUS);
+        wpavoid.clear();
+        loopv(waypoints) if(waypoints[i].weight() < 0) wpavoid.avoidnear(NULL, waypoints[i].o.z + WAYPOINTRADIUS, waypoints[i].o, WAYPOINTRADIUS);
     }
 
     struct wpcachestack
@@ -440,18 +440,18 @@ namespace ai
                 waypoints[d->ai->prevnodes[i]].curscore = -1;
                 waypoints[d->ai->prevnodes[i]].estscore = 0;
             }
-			if(retries <= 0)
-			{
-				loopavoid(obstacles, d,
-				{
-					if(iswaypoint(wp) && wp != node && wp != goal && waypoints[node].find(wp) < 0 && waypoints[goal].find(wp) < 0)
-					{
-						waypoints[wp].route = routeid;
-						waypoints[wp].curscore = -1;
-						waypoints[wp].estscore = 0;
-					}
-				});
-			}
+            if(retries <= 0)
+            {
+                loopavoid(obstacles, d,
+                {
+                    if(iswaypoint(wp) && wp != node && wp != goal && waypoints[node].find(wp) < 0 && waypoints[goal].find(wp) < 0)
+                    {
+                        waypoints[wp].route = routeid;
+                        waypoints[wp].curscore = -1;
+                        waypoints[wp].estscore = 0;
+                    }
+                });
+            }
         }
 
         waypoints[node].route = routeid;
@@ -474,7 +474,7 @@ namespace ai
                 if(iswaypoint(link) && (link == node || link == goal || waypoints[link].haslinks()))
                 {
                     waypoint &n = waypoints[link];
-                    int weight = max(n.weight, 1);
+                    int weight = max(n.weight(), 1);
                     float curscore = prevscore + n.o.dist(m.o)*weight;
                     if(n.route == routeid && curscore >= n.curscore) continue;
                     n.curscore = curscore;
@@ -508,11 +508,11 @@ namespace ai
     string loadedwaypoints = "";
     VARF(0, dropwaypoints, 0, 0, 1, if(dropwaypoints) getwaypoints());
 
-    int addwaypoint(const vec &o, int weight = -1)
+    int addwaypoint(const vec &o, int pull = -1)
     {
         if(waypoints.length() > MAXWAYPOINTS) return -1;
         int n = waypoints.length();
-        waypoints.add(waypoint(o, weight >= 0 ? weight : getweight(o)));
+        waypoints.add(waypoint(o, pull >= 0 ? pull : getpull(o)));
         invalidatewpcache(n);
         return n;
     }
@@ -542,21 +542,21 @@ namespace ai
     void inferwaypoints(gameent *d, const vec &o, const vec &v, float mindist)
     {
         if(!shouldnavigate()) return;
-    	if(shoulddrop(d) && !clipped(o) && !clipped(v))
-    	{
-			int from = closestwaypoint(o, mindist, false), to = closestwaypoint(v, mindist, false);
-			if(!iswaypoint(from)) from = addwaypoint(o);
-			if(!iswaypoint(to)) to = addwaypoint(v);
-			if(d->lastnode != from && iswaypoint(d->lastnode) && iswaypoint(from))
-				linkwaypoint(waypoints[d->lastnode], from);
-			if(iswaypoint(to))
-			{
-				if(from != to && iswaypoint(from) && iswaypoint(to))
-					linkwaypoint(waypoints[from], to);
-				d->lastnode = to;
-			}
-		}
-		else d->lastnode = closestwaypoint(v, CLOSEDIST, false);
+        if(shoulddrop(d) && !clipped(o) && !clipped(v))
+        {
+            int from = closestwaypoint(o, mindist, false), to = closestwaypoint(v, mindist, false);
+            if(!iswaypoint(from)) from = addwaypoint(o);
+            if(!iswaypoint(to)) to = addwaypoint(v);
+            if(d->lastnode != from && iswaypoint(d->lastnode) && iswaypoint(from))
+                linkwaypoint(waypoints[d->lastnode], from);
+            if(iswaypoint(to))
+            {
+                if(from != to && iswaypoint(from) && iswaypoint(to))
+                    linkwaypoint(waypoints[from], to);
+                d->lastnode = to;
+            }
+        }
+        else d->lastnode = closestwaypoint(v, CLOSEDIST, false);
     }
 
     void navigate(gameent *d)
@@ -575,22 +575,23 @@ namespace ai
                 if(!d->airmillis) linkwaypoint(waypoints[curnode], d->lastnode);
             }
             d->lastnode = curnode;
+            waypoints[d->lastnode].drag += curtime;
             if(d->ai && iswaypoint(prevnode) && d->lastnode != prevnode) d->ai->addprevnode(prevnode);
         }
         else if(!iswaypoint(d->lastnode) || waypoints[d->lastnode].o.squaredist(v) > dist*dist)
         {
             dist = RETRYDIST; // workaround
-			d->lastnode = closestwaypoint(v, dist, false);
+            d->lastnode = closestwaypoint(v, dist, false);
         }
     }
 
     void navigate()
     {
-    	if(shouldnavigate())
-    	{
-    	    navigate(game::player1);
-    	    loopv(players) if(players[i]) navigate(players[i]);
-    	}
+        if(shouldnavigate())
+        {
+            navigate(game::player1);
+            loopv(players) if(players[i]) navigate(players[i]);
+        }
         if(invalidatedwpcaches) clearwpcache(false);
     }
 
@@ -691,7 +692,7 @@ namespace ai
     {
         if(!mname || !*mname) mname = mapname;
         if(!*mname) return false;
-        formatstring(wptname)("%s.wpt", mname);
+        nformatstring(wptname, MAXSTRLEN, "%s.wpt", mname);
         path(wptname);
         return true;
     }
@@ -719,7 +720,7 @@ namespace ai
             o.x = f->getlil<float>();
             o.y = f->getlil<float>();
             o.z = f->getlil<float>();
-            waypoint &w = waypoints.add(waypoint(o, getweight(o)));
+            waypoint &w = waypoints.add(waypoint(o, getpull(o)));
             int numlinks = f->getchar(), k = 0;
             loopj(numlinks) if((w.links[k] = f->getlil<ushort>()) != 0) if(++k >= MAXWAYPOINTLINKS) break;
             //if(!w.haslinks()) conoutf("warning: waypoint %d has no links", i);
@@ -729,6 +730,7 @@ namespace ai
         conoutf("loaded %d waypoints from %s", numwp, wptname);
 
         if(!cleanwaypoints()) clearwpcache();
+        game::specreset();
         return true;
     }
     ICOMMAND(0, loadwaypoints, "s", (char *mname), if(!(identflags&IDF_WORLD)) getwaypoints(true, mname));
@@ -777,7 +779,7 @@ namespace ai
                 v.links[j] = k+1;
                 break;
             }
-            waypoint &w = waypoints.add(waypoint(v.o, getweight(v.o)));
+            waypoint &w = waypoints.add(waypoint(v.o, getpull(v.o)));
             int k = 0;
             loopvj(v.links) if((w.links[k] = v.links[j]) != 0) if(++k >= MAXWAYPOINTLINKS) break;
             //if(!w.haslinks()) conoutf("warning: imported waypoint %d has no links", i);
@@ -790,7 +792,8 @@ namespace ai
 
     bool getwaypoints(bool force, const char *mname, bool check)
     {
-        if(check && (client::demoplayback || loadedwaypoints[0])) return false;
+        //if(check && (client::demoplayback || loadedwaypoints[0])) return false;
+        if(check && loadedwaypoints[0]) return false;
         return loadwaypoints(force, mname) || importwaypoints();
     }
 
@@ -813,7 +816,7 @@ namespace ai
     void delselwaypoints()
     {
         if(noedit(true)) return;
-        vec o = sel.o.tovec().sub(0.1f), s = sel.s.tovec().mul(sel.grid).add(o).add(0.1f);
+        vec o = vec(sel.o).sub(0.1f), s = vec(sel.s).mul(sel.grid).add(o).add(0.1f);
         int cleared = 0;
         for(int i = 1; i < waypoints.length(); i++)
         {
@@ -837,7 +840,7 @@ namespace ai
     void moveselwaypoints(vec &offset)
     {
         if(noedit(true)) return;
-        vec o = sel.o.tovec().sub(0.1f), s = sel.s.tovec().mul(sel.grid).add(o).add(0.1f);
+        vec o = vec(sel.o).sub(0.1f), s = vec(sel.s).mul(sel.grid).add(o).add(0.1f);
         int moved = 0;
         for(int i = 1; i < waypoints.length(); i++)
         {
@@ -856,4 +859,3 @@ namespace ai
     }
     ICOMMAND(0, moveselwaypoints, "fff", (float *x, float *y, float *z), vec v(*x, *y, *z); moveselwaypoints(v));
 }
-
