@@ -234,6 +234,26 @@ struct packagedir
 vector<packagedir> packagedirs;
 int packagedirmask = ~0;
 
+int crcstream(stream *f)
+{
+    size_t len = 0;
+    char *buf = loadstream(f, &len, false);
+    if(!buf) return 0;
+    int crc = abs(int(crc32(0, (const Bytef *)buf, len)));
+    delete[] buf;
+    return crc;
+}
+
+int crcfile(const char *s)
+{
+    if(!s || !*s) return 0;
+    stream *f = openfile(s, "rb");
+    if(!f) return 0;
+    int crc = crcstream(f);
+    delete f;
+    return crc;
+}
+
 char *makefile(const char *s, const char *e, int revision, int start, bool store, bool skip)
 {
     static string o;
@@ -247,23 +267,25 @@ char *makefile(const char *s, const char *e, int revision, int start, bool store
         const char *q = t+1;
         if(isnumeric(*q)) d = min(atoi(q), 1);
     }
-    string f, m; copystring(f, o); copystring(m, o);
+    string f, m;
+    copystring(f, o);
+    copystring(m, o);
 
     for(bool tryrev = false;; skip = false)
     {
-        formatstring(m)("%s%s", f, *e ? e : "");
+        formatstring(m, "%s%s", f, *e ? e : "");
         if(skip || fileexists(findfile(m, "w"), "r"))
         {
             if(revision)
             {
                 if(!tryrev)
                 {
-                    formatstring(f)("%s%s.r%.4d", store ? "backups/" : "", o, revision);
+                    formatstring(f, "%s%s.r%.4d", store ? "backups/" : "", o, revision);
                     tryrev = true;
                 }
-                else formatstring(f)("%s%s.r%.4d.%.4d", store ? "backups/" : "", o, revision, d++);
+                else formatstring(f, "%s%s.r%.4d.%.4d", store ? "backups/" : "", o, revision, d++);
             }
-            else formatstring(f)("%s%s.%.4d", store ? "backups/" : "", o, d++);
+            else formatstring(f, "%s%s.%.4d", store ? "backups/" : "", o, d++);
         }
         else break;
     }
@@ -274,18 +296,19 @@ char *makefile(const char *s, const char *e, int revision, int start, bool store
 
 void backup(const char *fname, const char *ext, int revision, int start, bool store, bool full)
 {
-    defformatstring(tname)("%s%s", fname, ext);
-    defformatstring(aname)("%s", findfile(tname, "w"));
+    defformatstring(tname, "%s%s", fname, ext);
+    defformatstring(aname, "%s", findfile(tname, "w"));
     if(fileexists(aname, "r"))
     {
         const char *bname = aname;
         if(full) bname = findfile(makefile(fname, ext, revision, start, store), "w");
         else
         {
-            formatstring(tname)("%s%s.bak%s", store ? "backups/" : "", fname, ext);
+            formatstring(tname, "%s%s.bak%s", store ? "backups/" : "", fname, ext);
             bname = findfile(tname, "w");
         }
-        remove(bname); rename(aname, bname);
+        remove(bname);
+        rename(aname, bname);
     }
 }
 
@@ -307,7 +330,7 @@ char *makerelpath(const char *dir, const char *file, const char *prefix, const c
     if(cmd) concatstring(tmp, cmd);
     if(dir)
     {
-        defformatstring(pname)("%s/%s", dir, file);
+        defformatstring(pname, "%s/%s", dir, file);
         concatstring(tmp, pname);
     }
     else concatstring(tmp, file);
@@ -422,6 +445,11 @@ void sethomedir(const char *dir)
     fixdir(copystring(homedir, dir));
 }
 
+const char *gethomedir()
+{
+    return &homedir[0];
+}
+
 int maskpackagedirs(int flags)
 {
     int oldmask = packagedirmask;
@@ -441,7 +469,7 @@ const char *findfile(const char *filename, const char *mode)
     static string s;
     if(homedir[0])
     {
-        formatstring(s)("%s%s", homedir, filename);
+        formatstring(s, "%s%s", homedir, filename);
         path(s);
         if(fileexists(s, mode)) return s;
         if(mode[0]=='w' || mode[0]=='a')
@@ -460,15 +488,18 @@ const char *findfile(const char *filename, const char *mode)
             return s;
         }
     }
-    copystring(s, filename); path(s); // our own packages take priority
+    copystring(s, filename);
+    path(s); // our own packages take priority
     if(mode[0]=='w' || mode[0]=='a' || fileexists(s, mode)) return s;
     loopvrev(packagedirs) if((packagedirs[i].flags & packagedirmask) == packagedirs[i].flags)
     {
-        formatstring(s)("%s%s", packagedirs[i].name, filename); path(s);
+        formatstring(s, "%s%s", packagedirs[i].name, filename);
+        path(s);
         if(fileexists(s, mode)) return s;
     }
     if(mode[0]=='e') return NULL;
-    copystring(s, filename); path(s);
+    copystring(s, filename);
+    path(s);
     return s;
 }
 
@@ -476,7 +507,7 @@ bool listdir(const char *dirname, bool rel, const char *ext, vector<char *> &fil
 {
     size_t extsize = ext ? strlen(ext)+1 : 0;
 #ifdef WIN32
-    defformatstring(pathname)(rel ? ".\\%s\\*.%s" : "%s\\*.%s", dirname, ext ? ext : "*");
+    defformatstring(pathname, rel ? ".\\%s\\*.%s" : "%s\\*.%s", dirname, ext ? ext : "*");
     WIN32_FIND_DATA FindFileData;
     HANDLE Find = FindFirstFile(pathname, &FindFileData);
     if(Find != INVALID_HANDLE_VALUE)
@@ -498,7 +529,7 @@ bool listdir(const char *dirname, bool rel, const char *ext, vector<char *> &fil
         return true;
     }
 #else
-    defformatstring(pathname)(rel ? "./%s" : "%s", dirname);
+    defformatstring(pathname, rel ? "./%s" : "%s", dirname);
     DIR *d = opendir(pathname);
     if(d)
     {
@@ -536,12 +567,12 @@ int listfiles(const char *dir, const char *ext, vector<char *> &files)
     string s;
     if(homedir[0])
     {
-        formatstring(s)("%s%s", homedir, dirname);
+        formatstring(s, "%s%s", homedir, dirname);
         if(listdir(s, false, ext, files)) dirs++;
     }
     loopvrev(packagedirs) if((packagedirs[i].flags & packagedirmask) == packagedirs[i].flags)
     {
-        formatstring(s)("%s%s", packagedirs[i].name, dirname);
+        formatstring(s, "%s%s", packagedirs[i].name, dirname);
         if(listdir(s, false, ext, files)) dirs++;
     }
     dirs += listzipfiles(dirname, ext, files);
@@ -549,20 +580,20 @@ int listfiles(const char *dir, const char *ext, vector<char *> &files)
 }
 
 #ifndef STANDALONE
-static int rwopsseek(SDL_RWops *rw, int pos, int whence)
+static Sint64 rwopsseek(SDL_RWops *rw, Sint64 pos, int whence)
 {
     stream *f = (stream *)rw->hidden.unknown.data1;
     if((!pos && whence==SEEK_CUR) || f->seek(pos, whence)) return (int)f->tell();
     return -1;
 }
 
-static int rwopsread(SDL_RWops *rw, void *buf, int size, int nmemb)
+static size_t rwopsread(SDL_RWops *rw, void *buf, size_t size, size_t nmemb)
 {
     stream *f = (stream *)rw->hidden.unknown.data1;
     return f->read(buf, size*nmemb)/size;
 }
 
-static int rwopswrite(SDL_RWops *rw, const void *buf, int size, int nmemb)
+static size_t rwopswrite(SDL_RWops *rw, const void *buf, size_t size, size_t nmemb)
 {
     stream *f = (stream *)rw->hidden.unknown.data1;
     return f->write(buf, size*nmemb)/size;
@@ -1220,23 +1251,21 @@ stream *openutf8file(const char *filename, const char *mode, stream *file)
     return utf8;
 }
 
-char *loadfile(const char *fn, size_t *size, bool utf8)
+char *loadstream(stream *f, size_t *size, bool utf8)
 {
-    stream *f = openfile(fn, "rb");
-    if(!f) return NULL;
+    f->seek(0, SEEK_SET);
     size_t len = f->size();
-    if(len <= 0) { delete f; return NULL; }
+    if(len <= 0) return NULL;
     char *buf = new char[len+1];
-    if(!buf) { delete f; return NULL; }
+    if(!buf) return NULL;
     size_t offset = 0;
     if(utf8 && len >= 3)
     {
-        if(f->read(buf, 3) != 3) { delete f; delete[] buf; return NULL; }
+        if(f->read(buf, 3) != 3) { delete[] buf; return NULL; }
         if(((uchar *)buf)[0] == 0xEF && ((uchar *)buf)[1] == 0xBB && ((uchar *)buf)[2] == 0xBF) len -= 3;
         else offset += 3;
     }
     size_t rlen = f->read(&buf[offset], len-offset);
-    delete f;
     if(rlen != len-offset) { delete[] buf; return NULL; }
     if(utf8) len = decodeutf8((uchar *)buf, len, (uchar *)buf, len);
     buf[len] = '\0';
@@ -1244,3 +1273,11 @@ char *loadfile(const char *fn, size_t *size, bool utf8)
     return buf;
 }
 
+char *loadfile(const char *fn, size_t *size, bool utf8)
+{
+    stream *f = openfile(fn, "rb");
+    if(!f) return NULL;
+    char *buf = loadstream(f, size, utf8);
+    delete f;
+    return buf;
+}
